##########################################################################################
#
# JouePartie.tcl
#
#     Ce fichier contient l'ensemble des operations lies a la grille de jeu
#
##########################################################################################

namespace eval JouePartie {
    global Glob
    variable this
    
    ##########################################################################################
    # JouePartie::Protocole
    #
    #     Cette fonction realise les operations a effectuer en fonction des messages recus
    #     du serveur. Ces operations sont celles liees au protocole.
    #     
    ##########################################################################################
    proc Protocole {opt {args {}}} { Debug "[namespace current]::Protocole (opt=$opt args=$args)"
	global Glob
	variable this
	
	set args [lindex $args 0]
	switch [lindex $opt 0] {
	    "BAD"  {
		;# Le coup joue n'est pas le bon, alors on doit afficher le message 
		;# d'erreur recu du serveur
		Message "$Glob(prg) - Erreur" "$args" error
	    }
	    "PLAY" {
		;# Le serveur nous indique ici a quel joueur est le tour de jouer
		;# Nous devons donc afficher sur la fenetre de grille un message
		if {![winfo exists $this(wjoueurs)]} { return }
		set id_partie [lindex $args 0]
		set id_joueur [lindex $args 1]
		set msg [GetJoueur $id_partie $id_joueur]
		Debug "play: id_partie=$id_partie id_joueur=$id_joueur msg=$msg"
		$this(wmsg) configure -text "C'est a $msg de jouer !"
		bell
		if { $Glob(id_joueur) == $id_joueur } {
		    Actions "active"
		} 
	    }
	    "GRID" {
		;# Le serveur nous envoie l'ensemble des coups deja joues dans cette partie
		;# On efface donc tpus les bitmaps affiches et on replace les coups deja
		;# joues.
		;# Ceci permet de faire des mises a jours lors des QUIT des joueurs ou
		;# lorsque l'on integre une partie.
		if {![winfo exists $this(canvas)]} { return }
		set id_partie [lindex $args 0]
		Debug "grid: id_partie=$id_partie"
		$this(canvas) delete images
		foreach joueur [lrange $args 1 end] {
		    Debug "joueur=$joueur"
		    set j [lindex $joueur 0]
		    foreach coup [lrange $joueur 1 end] {
			Debug "coup=$coup"
			set x [lindex $coup 0]
			set y [lindex $coup 1]
			PutCoup $x $y $j
		    }
		}
	    }		
	    "HIT" {
		;# Le serveur conirme ainsi qu'il a bien accepte un coup joue par un joueur
		;# (nous ou un autre joueur de la partie)
		;# On affiche alors le bitmap correspondant a ce coup et joueur
		Actions "desactive"
		set x [lindex $args 1]
		set y [lindex $args 2]
		set j [lindex $args 3]
		PutCoup $x $y $j
	    }
	    "WIN" {
		;# Un des joueurs vient de gagner la partie, donc on doit le notifier
		;# par un message, en ensuite detruire la grille et retourner dans la
		;# phase d'attente de creation ou integration d'une partie.
		if {![winfo exists $this(wmsg)]} { return }
		$this(wmsg) configure -text "Fin de la partie !"
		set id_partie [lindex $args 0]
		set id_joueur [lindex $args 1]
		Debug "win: id_partie=$id_partie id_joueur=$id_joueur"
		set msg [GetJoueur $id_partie $id_joueur]
		Message "$Glob(prg)" "$msg\n  gagne la partie!" warning
		set Glob(phase) ListeParties
		ListeParties::Active
		destroy $this(top)
	    }
	    "END" {
		;# La grille est plein et il y a donc match nul On doit le notifier
		;# par un message, en ensuite detruire la grille et retourner dans la
		;# phase d'attente de creation ou integration d'une partie
		if {![winfo exists $this(wmsg)]} { return }
		$this(wmsg) configure -text "Fin de la partie !"
		Message "$Glob(prg)"  "Match nul !"  warning
		set Glob(phase) ListeParties
		ListeParties::Active
		destroy $this(top)
	    }
	}
    }

    ##########################################################################################
    # JouePartie::Draw
    #
    #     Cette fonction realise l'affichage de la fenetre contenant la grille de jeu et la
    #     liste des joueurs.
    #
    ##########################################################################################
    proc Draw {} { Debug "[namespace current]::Draw"
	global Glob
	variable this

	set w .partie
	set this(top) $w
	catch {destroy $w}
	toplevel $w
	wm title $w "$Glob(prg) - Partie en cours"
	wm iconname $w "Morpion"
	wm maxsize $w [winfo screenwidth .] [winfo screenheight .]
	wm resizable $w 0 0
	wm protocol $w WM_DELETE_WINDOW "Actions quit $w"

	set partie ""
	foreach p $Glob(parties) {
	    if {[lindex $p 0] == $Glob(id_partie)} {
		set partie $p
	    }
	}
	if {$partie != ""} {
	    set this(nom) [lindex $partie 1]
	    set this(lar) [lindex $partie 3]
	    set this(hau) [lindex $partie 4]
	} else {
	    set this(nom) $Glob(new_nom)
	    set this(lar) $Glob(new_lar)
	    set this(hau) $Glob(new_hau)
	}
	Debug "this(nom)=$this(nom) this(lar)=$this(lar) this(hau)=$this(hau)"
	
	;# La frame avec la grille est la liste des joueurs
	frame $w.h
	set this(canvas) [canvas $w.h.c -relief sunken -borderwidth 2 \
			      -width  [expr $this(lar)*$Glob(size)] \
			      -height [expr $this(hau)*$Glob(size)]]
	set this(wjoueurs) [ScrolledText $w.h.joueurs -width 40 -height 8]
	pack $w.h.c -side left -anchor n
	pack $w.h.joueurs -side right -fill both -expand yes

	;# La frame avec le bouton "Quitter Partie" et le message indiquant a qui
	;# est le tour de jouer
	frame $w.b
	set this(wmsg) [label $w.b.mes -text "Essai"]
	button $w.b.quit -text "Quitter la partie" \
		       -command "[namespace current]::Actions quit"
	pack $w.b.mes -side right -fill both -expand yes
	pack $w.b.quit -side left
	
	pack $w.h -side top -fill both -expand yes
	pack $w.b -side top -fill both

	update
	update idletasks
	;# On dessine la grille
	for {set i 0} {$i<$this(hau)} {incr i} {
	    for {set j 0} {$j<$this(lar)} {incr j} {
		$this(canvas) create rect \
		    [expr $j*$Glob(size)] [expr $i*$Glob(size)] \
		    [expr ($j+1)*$Glob(size)] [expr ($i+1)*$Glob(size)]\
		    -width 1 
	    }
	}
	;# On invalide les clics et double-clics afin d'eviter un bug de Tk.
	bind $this(wjoueurs) <1> { break }
	focus $this(canvas)
    }
    
    ##########################################################################################
    # JouePartie::PutCoup
    #
    #     Cette fonction permet d'afficher sur la grille un coup joue par un joueur.
    #     sous la forme de l'icone associe au joueur.
    ##########################################################################################
    proc PutCoup {x y j} { Debug "[namespace current]::PutCoup (x=$x y=$y j=$j)"
	global Glob
	variable this

	if {![winfo exists $this(canvas)]} { return }
	set half [expr $Glob(size) / 2]
	$this(canvas) create image \
			       [expr $x * $Glob(size) + $half] \
			       [expr $y * $Glob(size) + $half] \
			       -image $Glob(im$j) -tags images
    }

    ##########################################################################################
    # JouePartie::Update
    #
    #     Cette fonction realise la mise a jour de la liste des joueurs a chaque reception
    #     d'un message GAMES du serveur.
    #
    ##########################################################################################
    proc Update {} { Debug "[namespace current]::Update"
	global Glob
	variable this

 	if {[info exists this(wjoueurs)] && [winfo exists $this(wjoueurs)] && [info exists Glob(id_partie)]} {
	    
	    foreach partie $Glob(parties) {
		if {[lindex $partie 0] == $Glob(id_partie)} { break }
	    }
	    PutInfos $this(wjoueurs) $partie
	}
    }

    ##########################################################################################
    # JouePartie::GetJoueur
    #
    #     Cette fonction renvoi la chaine "nom_du_joueur sur machine_du_joueur" en fonction 
    #     de l'ID de la partie et de l'ID du joueur.
    #     Pour se faire, on analyse les donnees recues lors du precedent message GAMES
    #     provenant su serveur.
    #
    ##########################################################################################
    proc GetJoueur {id_partie id_joueur} { Debug "[namespace current]::GetJoueur (id_partie=$id_partie id_joueur=$id_joueur)"
	global Glob
	variable this

	foreach partie $Glob(parties) {
	    Debug "partie=$partie"
	    if {[lindex $partie 0] == $id_partie} {
		foreach joueur [lindex $partie 5] {
		    Debug "joueur=$joueur"
		    if {[lindex $joueur 0] == $id_joueur} {
			set nom [lindex [lindex $joueur 1] 0]
			set machine [lindex $joueur 2]
			Debug "$nom sur $machine"
			return "$nom sur $machine"
		    }
		}
	    }
	}
    }

    ##########################################################################################
    # JoueParties::Actions
    #
    #     Dans cette fonction on retrouve les differentes actions independantes du protocole
    #
    ##########################################################################################
    proc Actions {opt {args {}}} { Debug "[namespace current]::Actions ($opt, $args)"
	global Glob
	variable this

	switch $opt {
	    "active" {
		;# Cette action permet la saisie du coup du joueur lors d'un clic sur la grille
		if {[info exists this(canvas)] && [winfo exists $this(canvas)]} { 
		    bind $this(canvas) <1> "[namespace current]::Actions \"envoi_coup\" %x %y"
		    focus $this(canvas)
		}
	    }
	    "desactive" {
		;# Cette action invalide la saisie du coup du joueur lors d'un clic sur la grille
		if {[info exists this(canvas)] && [winfo exists $this(canvas)]} {
		    bind $this(canvas) <1> { }
		    focus $this(canvas)
		}
	    }
	    "envoi_coup" {
		;# Cette action permet de recuperer les coordonnees du coup du joueur
		;# et de les envoyer au serveur
		set x [expr [lindex $args 0] / $Glob(size)]
		set y [expr [lindex $args 1] / $Glob(size)]
		ecrit_socket "HIT $Glob(id_partie) $x $y\n" 
	    }
	    "quit" {
		;# Quand le joueur appuie sur le bouton "Quitter Partie", on lui demande de
		;# confirmer son depart, puis on detruit la grille et retourne dans la
		;# phase d'attente de creation ou integration d'une partie
		update
		set id [lindex $args 0]
		set r [tk_messageBox -icon warning -title "$Glob(prg)" -type yesno \
			   -message "Etes vous certain de vouloir quitter la partie ?" \
			   -parent $this(top)]
		switch $r {
		    "no" { return }
		    "yes" {
			set Glob(phase) ListeParties
			ecrit_socket "QUIT $Glob(id_partie)\n"
			ListeParties::Active
			destroy $this(top)
		    }
		}
	    }
	    
	}
    }

}
