##########################################################################################
#
# ListePartie.tcl
#
#      Ce fichier contient l'ensemble des operations lies a la phase d'identification
#      d'affichage des parties en cours, et d'integration d'une partie en cours
#
##########################################################################################

namespace eval ListeParties {

    variable this() {}

    ##########################################################################################
    # ListePartie::Protocole
    #
    #     Cette fonction realise les operations a effectuer en fonction des messages recus
    #     du serveur. Ces operations sont celles liees au protocole.
    #
    ##########################################################################################
    proc Protocole {opt {args {}}} { Debug "[namespace current]::Protocole (opt=$opt args=$args)"
	global Glob
	variable this
	
	set args [lindex $args 0] 
	switch [lindex $opt 0] {
	    "MSG" {
		;# On doit afficher dans la fenetre des messages les messages en envoyes par 
		;# le serveur c'est a dire toute la ligne suivant un MSG .....  
		$this(wmsg) configure -state normal
		$this(wmsg) insert end "$args\n"
		$this(wmsg) see end
		$this(wmsg) configure -state disabled	
	    }
	    "BAD" {
		;# On recoit un BAD si le surnom est refuse par le serveur.
		;# On doit alors afficher le message d'erreur, effacer le surnom saisi,
		;# et attendre un nouveau surnom
		Message "$Glob(prg) - Erreur" "[lindex $args 0]" error
		$this(went) delete 0 end
		focus $this(went)
	    }
	    "OK"  {
		;# On recoit un OK si le surnom est accepte par le serveur.
		;# On doit alors boquer la fenetre de saisie du surnom et autoriser
		;# les autres actions (Creation d'une nouvelle partie et integration d'une
		;# partie exstante).
		$this(went) configure -state disabled
		$this(bent) configure -state disabled
		pack forget $this(bent)
		ListeParties::Active
	    }
	    "JOIN" {
		;# Si un joueur demande a integrer une partie, on doit alors faire apparaitre
		;# une boite de dialogue permettant d'accepter ou refuser l'acces a la partie
		;# pour ce joueur, et renvoyer au serveur un message d'acceptation ou refus.
		set id_partie  [lindex $args 0]
		set nom_joueur [lindex $args 1]
		foreach partie $Glob(parties) {
		    if {[lindex $partie 0] == $id_partie} {
			set nom_partie [lindex $partie 1]
		    }
		}
		set res [tk_messageBox -icon error -title "$Glob(prg)" -type yesno \
			     -message "Acceptez-vous $nom_joueur dans $nom_partie"]
		switch $res {
		    "yes" { ecrit_socket "YES $id_partie $nom_joueur\n"
		    }
		    "no" { ecrit_socket "NO $id_partie $nom_joueur\n"
		    }
		}
	    }
	}
    }

    ##########################################################################################
    # ListePartie::Draw
    #
    #      Cette fonction realise l'affichage de la fenetre contenant la liste des parties,
    #      la liste des informations sur la partie cliquee par le joueur, la fenetre des
    #      messages informatifs envoyes par le serveur.
    #
    ##########################################################################################
    proc Draw {} { Debug "[namespace current]::Draw"
	global Glob
	variable this

	set w .parties
	set this(top) $w
	catch {destroy $w}
	toplevel $w
	wm title $w "$Glob(prg) - Liste des parties en cours"
	wm iconname $w $Glob(prg)
	wm maxsize $w [winfo screenwidth .] [winfo screenheight .]
	wm minsize $w 100 100
	wm resizable $w 0 0
	wm protocol $w WM_DELETE_WINDOW "Actions quit $w"

	;# La fenetre des messages informatifs (message du jour)
	set this(wmsg) [ScrolledText $w.t -width 40 -height 5 -wrap word -state disabled]
	
	;# La fenetre de saisie du surnom
	frame $w.top
	set this(went) [LabEntry $w.top.e -text "Surnom" -textvariable Glob(nick)]
	set this(bent) [button $w.top.b -text "OK" \
			    -command "[namespace current]::Actions send_nick"]
	pack $w.top.e $w.top.b -side left

	;# La fenetre avec la liste des parties et celles des informations
	frame $w.h
	set this(wparties) [ScrolledListbox $w.h.parties -width 20 -height 6 \
				-selectmode single]
	set this(winfos) [ScrolledText $w.h.infos -width 50 -height 6 -state disabled]
	pack $w.h.parties $w.h.infos -side left -fill both -expand yes

	;# Les bouton du bas
	frame $w.b
	set this(bnew)  [button $w.b.new    -text "Nouvelle Partie" -width 17 \
			     -state disabled \
			     -command "[namespace current]::Actions new"]
	set this(bjoin) [button $w.b.join   -text "Joindre Partie"  -width 17 \
			     -state disabled \
			     -command "[namespace current]::Actions join"]
	button $w.b.cancel -text "Quitter"         -width 17 \
		       -command "[namespace current]::Destroy"
	pack $w.b.new $w.b.join $w.b.cancel -side left -padx 15 -fill both -expand yes

	pack $w.t -side top -fill both -expand yes
	pack $w.top -side top -anchor w
	pack $w.h -side top -fill both -expand yes
	pack $w.b -side top -fill both

	update
	;# On invalide les clics et double-clics afin d'eviter un bug de Tk.
	bind $this(wmsg)   <1> { break }
	bind $this(winfos) <1> { break }
	bind $this(went)   <1> { break }
	;# Un "Return" provoque l'envoi du surnom
	bind $this(top) <Return> "$w.top.b invoke"
	grab $this(top)
	focus $this(went)
	;# Le surnom doit etre un seul mot en  alphanumerique
	trace variable Glob(nick) w "Force AlphaNum"
	set Glob(nick) ""
    }

    ##########################################################################################
    # ListePartie::Destroy
    #
    #      
    ##########################################################################################
    proc Destroy {} { Debug "[namespace current]::Destroy"
	global Glob
	variable this

	update
	set r [tk_messageBox -icon warning -title "$Glob(prg)" -type yesno \
		   -message "Etes vous certain de vouloir quitter le jeu ?" \
		   -parent $this(top)]
	switch $r {
	    "no" { return }
	    "yes" { destroy $this(top)
		ferme_connexion
		exit
	    }
	}
    }

    ##########################################################################################
    # ListePartie::Active
    #
    #      Cette fonction remet en mode normal les boutons et bindings sur les fenetres
    #      autorisant ainsi le passage en NewPartie ou JoinPartie
    #
    ##########################################################################################
    proc Active {} { Debug "[namespace current]::Active"
	global Glob
	variable this
	
	$this(bnew) configure -state normal
	$this(bjoin) configure -state normal
	update
	focus $this(bnew)
	bind $this(top) <Return> "$this(bnew) invoke"
	bind $this(wparties) <Double-1> "$this(bjoin) invoke"
    }

    ##########################################################################################
    # ListePartie::Desactive
    #
    #      Cette fonction supprime l'effet des boutons et bindings sur les fenetres
    #      empechant ainsi le passage en NewPartie ou JoinPartie
    #      
    ##########################################################################################
    proc Desactive {} { Debug "[namespace current]::Desactive"
	global Glob
	variable this

	$this(bnew) configure -state disabled
	$this(bjoin) configure -state disabled
	update
	bind $this(top) <Return> {}
	bind $this(wparties) <Double-1> {}
    }

    ##########################################################################################
    # ListeParties::Update
    #
    #     Cette fonction realise la mise a jour de la liste des parties a chaque reception
    #     d'un message GAMES du serveur.
    #
    ##########################################################################################
    proc Update {} { Debug "[namespace current]::Update"
	global Glob
	variable this

	$this(wparties) delete 0 end
	Debug "Glob(parties)=$Glob(parties)"
	if {[llength $Glob(parties)]!=0} {
	    foreach partie $Glob(parties) {
		set nom [lindex $partie 1]
		set nb  [llength [lindex $partie 5]]
		$this(wparties) insert end "$nom ($nb joueurs)"
	    }
	    if {$Glob(phase)=="ListeParties" } {
		$this(bjoin) configure -state normal
	    }
	    bind $this(wparties) <1> "[namespace current]::ShowInfos %x %y"
	    Debug "bindings=[bind $this(wparties)]"
	} else {
	    $this(wparties) insert end "Aucune partie en cours"
	    $this(winfos) configure -state normal
	    $this(winfos) delete 0.0 end
	    $this(winfos) configure -state disabled
	    $this(bjoin) configure -state disabled
	    bind $this(wparties) <1> {}
	}
    }

    ##########################################################################################
    # ListeParties::ShowInfos
    #
    #      Cette fonction realise la mise a jour de la fenetre d'informations sur la partie
    #      lorsque l'utilisateur clique sur une partie.
    #
    ##########################################################################################
    proc ShowInfos { x y } { Debug "[namespace current]::ShowInfos (x=$x y=$y)"
	global Glob
	variable this

	set n [$this(wparties) index @$x,$y]
	set partie [lindex $Glob(parties) $n]
	PutInfos $this(winfos) $partie
    }

    ##########################################################################################
    # ListeParties::Actions
    #
    #      Dans cette fonction on retrouve les differentes actions independantes du protocole
    ##########################################################################################
    proc Actions { opt {args ""}} { Debug "[namespace current]::Actions ($opt, $args)"
	global Glob
	variable this

	switch $opt {
	    "send_nick" {
		;# Ici on relaise l'envoi du surnom au serveur
		ecrit_socket "NICK $Glob(nick)\n"
	    }
	    "new" {
		;# Permet de passer dans la phase NewPartie si l'on appuie sur le 
		;# bouton "Nouvelle Partie"
		ListeParties::Desactive
		NewPartie::Draw
		set Glob(phase) NewPartie
	    }
	    "join" {
		;# Permet de passer dans la phase JoinPartie si on double-clique dans la
		;# liste des parties ou si l'on appuie sur le bouton "Joindre Partie"
		set n [$this(wparties) curselection]
		if { $n != ""} {
		    set Glob(id_partie) [lindex [lindex $Glob(parties) $n] 0]
		    ecrit_socket "JOIN $Glob(id_partie)\n"
		    Desactive
		    JoinPartie::Draw
		    set Glob(phase) "JoinPartie"
		}
	    }
	}
    }
    
}