##############################################################################################
#
# NewPartie.tcl
#
#    Ce fichier contient l'ensemble des operations lies a la saisie et acceptation
#    des parametres de creation d'une nouvelle partie. 
#
##############################################################################################

namespace eval NewPartie {
    global Glob
    variable this

    ##########################################################################################
    # NewPartie::Protocole
    #
    #    Cette fonction realise les operations a effectuer en fonction des messages recus
    #    du serveur. Ces operations sont celles liees au protocole.
    #     
    ##########################################################################################
    proc Protocole {opt {args {}}} { Debug "[namespace current]::Protocole"
	global Glob
	variable this
	
	set args [lindex $args 0]
	switch [lindex $opt 0] {
	    "BAD" { 
		;# On recoit un BAD si au moins un des parametres de creation d'une nouvelle 
		;# partie est incorrect
		;# Dans ce cas on affiche un message et l'utilisateur doit recommencer la 
		;# saisie
		tk_messageBox -icon error -title "$Glob(prg)" -type ok \
		    -message "$args"
		set Glob(new_nom) ""
		set Glob(new_hau) ""
		set Glob(new_lar) ""
		focus $this(we1)
	    }
	    "OK"  {
		;# Si on recoit un OK c'est que la creation de la nouvelle partie a ete
		;# faite par le serveur. On doit alors dessiner la fenetre de jeu et passer
		;# en phase de jeu (phase JouePartie)
		set Glob(id_partie) [lindex $args 0]
		grab release $this(top)
		ListeParties::Desactive
		destroy $this(top)
		set Glob(phase) JouePartie
		JouePartie::Draw
	    }
	}
    }
    
    ##########################################################################################
    # NewPartie::Draw
    #
    #    Cette fonction realise l'affichage de la fenetre de saisie des parametres pour la
    #    creation d'une nouvelle partie.
    #
    ##########################################################################################
    proc Draw {} { Debug "[namespace current]::Draw"
	global Glob verrou
	variable this
	
	set w .nouv_partie
	set this(top) $w
	catch {destroy $w}
	toplevel $w
	wm title $w "$Glob(prg) - Nouvelle Partie"
	wm iconname $w $Glob(prg)
	wm maxsize $w [winfo screenwidth .] [winfo screenheight .]
	wm resizable $w 0 0
	
	;# La frame de saisie des parametres
	frame $w.h
	label $w.h.lab -text "Entrez les informations sur la nouvelle partie"	
	set this(we1) [LabEntry $w.h.le1 -text "Nom :" -lwidth 8 \
			   -textvariable Glob(new_nom)]
	LabEntry $w.h.le2 -text "Largeur :"    -lwidth 8 -textvariable Glob(new_lar)
	LabEntry $w.h.le3 -text "Hauteur :"    -lwidth 8 -textvariable Glob(new_hau)
	LabRadio $w.h.rad1 -text "Morpion"     -width 10 -variable Glob(new_type) -value 0
	LabRadio $w.h.rad2 -text "Puissance4"  -width 10 -variable Glob(new_type) -value 1
	
	pack $w.h.lab -fill x -expand yes
	pack $w.h.le1 $w.h.le2 $w.h.le3 -side top -fill x
	pack $w.h.rad1 $w.h.rad2 -side top

	;# La frame avec les boutons
	frame $w.b
	button $w.b.new -text "OK" -width 7 \
		       -command "[namespace current]::Actions envoi_new"
	button $w.b.cancel -text "Annuler" \
		       -width 7 -command "[namespace current]::Actions quit"
	pack $w.b.new $w.b.cancel -side left -padx 15 -fill both -expand yes

	pack $w.h -side top -fill both -expand yes
	pack $w.b -side top -fill both

	centerDialog $this(top) $ListeParties::this(top)

	update
	bind $this(top) <Return> "$w.b.new invoke"
	grab $this(top)
	update
	focus $this(we1)

	;# Le nom de la partie doit etre un seul mot en AlphaNumerique
	trace variable Glob(new_nom) w "Force AlphaNum"
	;# La taille de la grille est exprime en entiers
	trace variable Glob(new_lar) w "Force Int"
	trace variable Glob(new_hau) w "Force Int"
	set Glob(new_nom) ""
	set Glob(new_hau) ""
	set Glob(new_lar) ""
	set Glob(new_type) 0
    }
    
    ##########################################################################################
    # NewParties::Actions
    #
    #    Dans cette fonction on retrouve les differentes actions independantes du protocole
    #
    ##########################################################################################
    proc Actions {opt {args {}}} { Debug "[namespace current]::Actions"
	global Glob 
	variable this

	switch $opt {
	    "envoi_new" {
		;# Cette action permet d'envoyer au serveur les parametres de la partie a creer
		ecrit_socket "NEW $Glob(new_nom) $Glob(new_lar) $Glob(new_hau) $Glob(new_type)\n" 
	    }
	    "quit" {
		;# Si l'on quitte cette phase on doit redonner la main a la phase ListeParties
		grab release $this(top)
		ListeParties::Active
		destroy $this(top)
		set Glob(phase) ListeParties
	    }
	}
    }
    
}
