/***************************************************************************
/*   Project   : TitBet (morpion en reseau)
/*   File Name : $Id$
/*   Author    : Pierre VASSELLERIE 
/*
/*   Begining : 15 Mars 1998
/*
/****************************************************************************
/*   History: 
/*            $Log$        
/***************************************************************************/

#include "config.h"
#include "global.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef HAVE_ERRNO_H
#  include <errno.h>
#endif

#ifdef HAVE_NET_ERRNO_H
#  include <net/errno.h>
#endif

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif

#include <tcl.h>
#include <tk.h>

#ifdef HAVE_SYS_SOCKET_H
#  include <sys/socket.h>
#endif

#ifdef HAVE_NETINET_IN_H
#  include <netinet/in.h>
#endif

#ifdef HAVE_ARPA_INET_H
#  include <arpa/inet.h>
#endif

#ifdef HAVE_NETDB_H
#  include <netdb.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#  include <sys/types.h>
#endif

#ifdef  HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif

#ifdef HAVE_TIME_H
#  include <time.h>
#  ifdef TIME_WITH_SYS_TIME
#    include <sys/time.h>
#  endif
#endif

#ifdef HAVE_LINUX_TIME_H
#  include <linux/time.h>
#endif



/**********************************************************************************************/
#include "client.h"


/**********************************************************************************************/
/* Ceci permet l'affichage des messages de debug si la compilation est
/* effectuee avec l'option -DDEBUG
/**********************************************************************************************/
#ifdef DEBUG
#define Debug   printf
#else
#define Debug
#endif


/**********************************************************************************************/
/* On definit ici le tableau associant les nouvelles commandes Tcl aux
/* fonctions de cette librairie
/**********************************************************************************************/
typedef struct _tcl_cmds
{
  char 		tcl_commande [50];
  int 	 	(*fonction) ( void*, Tcl_Interp*, int, char** );
} TclCmd;


static TclCmd Matrice_Cmds [] = 
{
  { "ouvre_connexion", 	                 &OuvreConnexion},
  { "ferme_connexion", 	                 &FermeConnexion},
  { "ecrit_socket", 	                 &EcritSocket},
  { "lit_socket", 	                 &LitSocket},
};


/**********************************************************************************************/
/* Variables globales
/**********************************************************************************************/
int          Socket;		/* le handler de la socket utilisee */
int          Connected;		/* flag permettant de savoir si l'on est connecte */
Tcl_DString  Message;		/* le buffer des messages recus */

extern int h_errno;		/* retourne l'errno du gethostbyname */


/**********************************************************************************************/
/* Cette fonction est appele automatiquement par l'interpreteur wish ou
/* tclsh lors du chargement de la librairie dynamique libsocket
/* Elle permet d'enregistrer les nouvelles commandes aupres de l'interpreteur
/**********************************************************************************************/
int Titbet_Init ( Tcl_Interp *interp )
{
  int	i;
  
  if (Tcl_PkgProvide (interp, "socket", "1.0") != TCL_OK) 
    return TCL_ERROR;
  
  for (i=0; i<sizeof(Matrice_Cmds)/sizeof(TclCmd); i++)
    {
      Tcl_CreateCommand(interp, Matrice_Cmds[i].tcl_commande,
			Matrice_Cmds[i].fonction,
			(ClientData)NULL,
			(Tcl_CmdDeleteProc*)NULL);
    };

  Connected=0;	/* Pour l'instant il n'y a pas de connexion */
  /* Initialisation du buffer des messages recus */
  Tcl_DStringInit(&Message);

  return TCL_OK;
}


/**********************************************************************************************/
/*  OuvreConnexion
/*  Tcl: ouvre_connexion serveur no_port
/*  
/*  Realise la connexion au serveur+no_port specifie et sinon renvoie un message
/*  d'erreur donnant la raison de l'echec de la connexion
/*
/**********************************************************************************************/
int OuvreConnexion(ClientData thisPtr, Tcl_Interp* interp, int argc, char** argv)
{
  char                   *hostname;
  unsigned short         port;
  struct hostent         *host;
  struct sockaddr_in     server_addr;
  int                    err;

  Debug("OuvreConnexion\n");
  Tcl_ResetResult(interp);
  /* On verifie que l'on a le bon nombre d'arguments */
  if (argc != 3) {
      Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
                       " serveur no_port\"", (char *) NULL);
      return TCL_ERROR;
    }

  /* On verifie tout de suite que la connexion n'existe pas deja (ca ne doit jamais arrive) */
  Debug("Verification connexion deja existante\n");
  if (Connected) {
    Tcl_SetResult(interp, "La connexion est deja realisee.", TCL_VOLATILE);
    return TCL_ERROR;
  }

  /* Recuperation des arguments de la commande */
  hostname=(char*) strdup(argv[1]);
  port=(atoi(argv[2]) & 0xFFFF);
  Debug("hostname=%s port=%d\n", hostname, port);

  /* Recuperation de l'adresse IP du serveur */
  Debug("gethostbyname\n");
  host=gethostbyname(hostname);
  if (host== NULL) {
    switch(h_errno) {
    case HOST_NOT_FOUND : 
      Tcl_SetResult(interp, "Serveur inconnu.", TCL_VOLATILE);
      return TCL_ERROR;
    case NO_ADDRESS : 
      Tcl_SetResult(interp, "Il n'y a pas d'adresse IP associee a ce nom.", TCL_VOLATILE);
      return TCL_ERROR;
    case NO_RECOVERY : 
      Tcl_SetResult(interp, "Erreur fatale du serveur de noms.", TCL_VOLATILE);
      return TCL_ERROR;
    case TRY_AGAIN : 
      Tcl_SetResult(interp, "Erreur du serveur de nom. Reessayez plus tard.", TCL_VOLATILE);
      return TCL_ERROR;
    default :
      Tcl_SetResult(interp, "Erreur inconnue.", TCL_VOLATILE);
      return TCL_ERROR;
   }
  }
  free(hostname);
  
  /* Initialisation de l'adresse, du port et de la famille de socket a utiliser */
  Debug("Initialisation adresse+port+famille\n");

#ifdef HAVE_BZERO
  bzero((char*) &server_addr, sizeof(server_addr));
#else
  memset((char*) &server_addr, 0, sizeof(server_addr));
#endif

#ifdef HAVE_BCOPY
  bcopy(host->h_addr, (char*) &server_addr.sin_addr, host->h_length);
#else
  memcpy (&server_addr.sin_addr, host->h_addr, host->h_length);  
#endif

  server_addr.sin_family = AF_INET;
  server_addr.sin_port   = htons(port);

  /* Creation de la socket */
  Socket=socket(AF_INET, SOCK_STREAM, 0);
  if (Socket<0) {
    Tcl_SetResult(interp, "Creation de la socket de communication impossible.", TCL_VOLATILE);
    return TCL_ERROR;
  }
  
  /* Ouverture de la connexion avec le serveur */
  Debug("Ouverture de la connexion\n");
  err=connect(Socket, (struct sockaddr *) &server_addr, sizeof(server_addr));
  if (err<0) {
    close(Socket);
    Tcl_SetResult(interp, "Connexion au serveur impossible.\n", TCL_VOLATILE);
    switch (errno) {
    case ECONNREFUSED :
      Tcl_AppendResult(interp, "Pas de serveur de morpion sur cette machine.", (char*) NULL);
      return TCL_ERROR;
    case ETIMEDOUT : 
      Tcl_AppendResult(interp, "Depassement du delai maximum pendant la  connexion.", (char*) NULL);
      return TCL_ERROR;
    case ENETUNREACH : 
      Tcl_AppendResult(interp, "Le reseau est inaccessible.", (char*) NULL);
      return TCL_ERROR;
    case EHOSTUNREACH : 
      Tcl_AppendResult(interp, "Le serveur ne peut pas etre joint.", (char*) NULL);
      return TCL_ERROR;
    default :
       Tcl_AppendResult(interp, "Erreur inconnue.", (char*) NULL);
       return TCL_ERROR;
    }
  }
  Connected=1;
  return TCL_OK;
} 

/**********************************************************************************************/
/*  FermeConnexion
/*  Tcl: ferme_connexion
/*  
/*  Ferme proprement la connexion avec le serveur
/*
/**********************************************************************************************/
int FermeConnexion(ClientData thisPtr, Tcl_Interp* interp, int argc, char** argv)
{
  Debug("FermeConnexion (%d)\n", argc);
  Tcl_ResetResult(interp);
  
  /* On verifie tout de suite que la connexion existe deja */
  Debug("Verification connexion existante\n");
  if (!Connected) {
    Tcl_SetResult(interp, "La connexion n'existe pas.", TCL_VOLATILE);
    return TCL_ERROR;
  }

  shutdown(Socket, 2);
  close(Socket);
  Connected=0;
  Debug("Deconnexion\n");
  return TCL_OK;
}

/**********************************************************************************************/
/*  EcritSocket
/*  Tcl:  ecrit_socket "message a envoyer"
/*  
/*  Envoi le message passe en argument au serveur sur la connexion existante 
/*  (le tranfert peut se faire en plusieurs fois)
/*
/**********************************************************************************************/
int EcritSocket(ClientData thisPtr, Tcl_Interp* interp, int argc, char** argv)
{
  char         *p;
  unsigned     msg_len;
  int          reste, n;

  Debug("EcritSocket (%d)\n", argc);
  Tcl_ResetResult(interp);
  /* On verifie que l'on a le bon nombre d'arguments */
  if (argc != 2) {
      Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
                       "message \"", (char *) NULL);
      return TCL_ERROR;
    }

  /* On verifie tout de suite que la connexion existe deja */
  Debug("Verification connexion existante\n");
  if (!Connected) {
    Tcl_SetResult(interp, "La connexion n'existe pas.", TCL_VOLATILE);
    return TCL_ERROR;
  }

  Debug("Message a envoyer=|%s|\n", argv[1]);

  p=argv[1];
  msg_len=strlen(argv[1]);
  reste=msg_len;

  while(reste>0) {
    n=write(Socket, p, reste);
    if (n<0) {
      Tcl_SetResult(interp, "Erreur lors de l'ecriture dans la socket.", TCL_VOLATILE);
      return TCL_ERROR;
    }
    Debug("Envoi de %d caracteres\n", n);
    reste-=n;
    p+=n;
  }

  Debug("Envoi de %d sur %d caracteres termine\n", msg_len-reste, msg_len);
  return TCL_OK;
}

/**********************************************************************************************/
/*  SocketReadable
/*  
/*  Cette fonction realise un select durant un certain delai (voir definition
/*  dans client.h) afin de regarder si quelque chose est disponible en lecture
/*  sur la socket.
/*
/**********************************************************************************************/
int SocketReadable()
{
    int                 readfds;
    struct timeval      timeout;

#ifndef timerclear
#define timerclear(tvp)   (tvp)->tv_sec = (tvp)->tv_usec = 0
#endif

    timerclear(&timeout);
    timeout.tv_sec = TIMEOUT_SEC;
    timeout.tv_usec = TIMEOUT_USEC;
    readfds = (1 << Socket);

    if (select(Socket + 1, &readfds, NULL, NULL, &timeout) == -1)
      return ((errno == EINTR) ? 0 : -1);
    
    if (readfds & (1 << Socket)) return (1);
      return (0);
}

/**********************************************************************************************/
/*  OuEst
/*    args: 1) un pointeur sur une chaine
/*          2) un caractere
/*
/*  Renvoie l'index de la premiere occurrence du caractere dans la chaine
/*
/**********************************************************************************************/
int OuEst(char *s, char c) 
{
  int i;

  for(i=0; s[i]!=c && s[i]!='\0'; i++);
  if (s[i]=='\0') return -1;
    else return i;
}


/**********************************************************************************************/
/*  LitSocket
/*  Tcl: lit_socket
/*  
/*  Realise la lecture des donnees presentes sur la socket de la connexion avec le serveur,
/*  les concatene aux donnees deja recues, et renvoi la premiere ligne du message (delimitee 
/*  par un retour chariot '\n') des que celle-ci est disponible.
/*  Si les donnees lues representent plus qu'une ligne de message alors les donnees
/*  restantes sont stockees dans le buffer dynamique "message" et seront disponible
/*  au prochain appel de la fonction.
/*
/**********************************************************************************************/
int LitSocket(ClientData thisPtr, Tcl_Interp* interp, int argc, char** argv)
{
  int          n, l;
  char         buf[1002];
  char         *p;

  Tcl_ResetResult(interp);

  if (!Connected) 
    return TCL_OK;	

  /* S'il n'y a aucun message disponible, alors on repart */
  if (!SocketReadable() && (Tcl_DStringLength(&Message)==0)) {
    Tcl_SetResult(interp, "", TCL_VOLATILE);
    return TCL_OK;
  }

  /* On lit les donnees disponibles sur la socket */
  if (SocketReadable()) { 
    n=read(Socket, buf, 1000);
    Debug("n=%d\n", n);
    switch (n) {
    case -1 : /* Erreur */
      Tcl_SetResult(interp, "Erreur lors de la lecture dans la socket.", TCL_VOLATILE);
      return TCL_ERROR;
    
    case 0 : /* Rien sur la socket <-> le serveur est tombe */
      shutdown(Socket, 2);
      close(Socket);
      Connected=0;
      Tcl_SetResult(interp, "Le serveur a coupe la connexion !", TCL_VOLATILE);
      return TCL_ERROR;

    default : /* On a lu quelque chose */
      buf[n]='\0';
      Debug("n=%d buf=>%s<\n", n, buf);
      /* On ajoute ce que l'on vient de lire au message precedent */
      Tcl_DStringAppend(&Message, buf, n);
      Debug("Message=|%s|\n", Tcl_DStringValue(&Message));
    }
  }

  /* On cherche le 1er retour chariot */
  l=OuEst(Tcl_DStringValue(&Message), '\n');
  Debug("l=%d\n", l);
  if (l!=-1)
    Tcl_SetObjResult(interp, Tcl_NewStringObj(Tcl_DStringValue(&Message), l));
  
  /* On reinitialise le buffer pour le message restant */
  p=(char*) strdup(Tcl_DStringValue(&Message));
  n=Tcl_DStringLength(&Message);
  Tcl_DStringFree(&Message);
  Tcl_DStringInit(&Message);
  if (n-l-1>0)
    Tcl_DStringAppend(&Message, p+l+1, n-l-1);
  
  return TCL_OK;
}
