#!/bin/sh
###########################################################################
#
#  Project : TitBet (un morpion en reseau)
#  Author  : Pierre Vassellerie (Pierre.Vassellerie@obspm.fr)
#  Date    : $Id$
#
###########################################################################
# la prochaine ligne lance wish8.0 et interprete la suite  \
    exec wish8.0 "$0" "$@"

# On recupere le path du binaire afin de savoir ou aller chercher les
# sources et librairie
set pwd "[exec dirname $argv0]/../lib"

# Chargement de l'extension contenant les commandes sur les sockets
# (si on utilise l'interpreteur titbetsh alors on n'a pas a charger
# les nouvelles fonctions)
if {[info commands ouvre_connexion] == ""} {
	set ext [info sharedlibextension]
	load $pwd/libtitbet$ext
}

# Chargement des definitions des couleurs et des autres ressources X11
# suivant si l'ecran est monochrome ou couleur
if {[winfo screendepth .] > 7 } { 
	source $pwd/options.tcl
}

# Chargement de la librairie de commandes utiles
source $pwd/commands.tcl
# Chargement de la librairie de widgets utiles
source $pwd/widgets.tcl 
# Inclusion des differents sources du morpion
source $pwd/ListeParties.tcl
source $pwd/NewPartie.tcl
source $pwd/JoinPartie.tcl
source $pwd/JouePartie.tcl


#
# Variables globales
#
set Glob(prg)  "TitBet (1.6)"
set Glob(size) 25


###########################################################################
#
# Fonction servant a autorise ou non l'affichage des messages de debug
# (un # en debut du ligne du puts supprime l'affichage)
#
###########################################################################
proc Debug { msg {args {}}} {
	puts stderr "$msg $args"
}

###########################################################################
#
# Fonction realisant l'affichage des messages d'erreurs
#
###########################################################################
proc bgerror { msg } { global Glob errorInfo

	after idle {
		.dialog999.msg configure -wraplength 4i
		if {$errorInfo==""} {
		  .dialog999.button1 configure -state disabled
		}	 
	}
	set i [tk_dialog .dialog999 "Erreur Fatale - $Glob(prg)" $msg \
	error 0 OK Trace]
	if {$i==1 && $errorInfo!=""} { puts stderr "\nTrace: $errorInfo\n" }
	exit
}

###########################################################################
#
# Fonction d'initialisation des icones associes aux joueurs
#
# (les id des joueurs commence a 5)
# (on commence a 4 car les joueurs n'etant plus presents on un id egal a 4) 
###########################################################################
proc InitIcones { dir } { global Glob
	
	set i 0
	foreach f [glob $dir/*.gif] {
  	  image create photo image$i -file $f 
  	  set Glob(im$i) image$i
  	  incr i
	}
}

###########################################################################
#
# Fonction realisant l'affichage des informations sur une partie dans
# une fenetre de type text.
#
###########################################################################
proc PutInfos { w partie } { Debug "PutInfos (w=$w id=$partie)"
    global Glob

    if {[winfo exists $w]} {
	set nom  [lindex $partie 1]
	set type [lindex $partie 2]
	set lar  [lindex $partie 3]
	set hau  [lindex $partie 4]
	
	$w tag configure titre -underline on -font {Helvetica 16 bold} \
	    -justify center
	$w tag configure joueur -font {Times 10 bold}  
	
	$w configure -state normal
	$w delete 0.0 end
	$w insert end "$nom\n" titre
	$w insert end "Type: $type\n"
	$w insert end "Taille: $lar x $hau\n"
	$w insert end "Joueurs:\n"
	foreach joueur [lindex $partie 5] {
	    set id      [lindex $joueur 0]
	    set nom     [lindex $joueur 1]
	    set machine [lindex $joueur 2]
	    if { $nom == $Glob(nick) } { set Glob(id_joueur) $id }
	    $w insert end " "
	    if { $nom == $Glob(nick) } {
		set Glob(id_joueur) $id
		$w insert end "> " joueur
	    } else {
		$w insert end "  " joueur
	    }
	    $w image create end -image $Glob(im$id)
	    $w insert end " $nom sur $machine\n" joueur
	}		    
	$w configure -state disabled
	update
    }
}

###########################################################################
#
# Fonction analysant les messages du protocole en entree du client
#
###########################################################################
proc TraiteProtocole { msg } { global Glob

	if {$msg != ""} {
		Debug "TraiteProtocole: msg=|$msg|([string first " " $msg])"
		set args [string range $msg [expr [string first " " $msg] +1]  end]
		set opt [lindex $msg 0]
		Debug "opt=$opt args=$args"

		switch $opt {
			"OK" -
			"BAD" { $Glob(phase)::Protocole $opt $args
			}
			"GAMES" {
				set Glob(parties) [lindex $args 0]
				ListeParties::Update
				JouePartie::Update
			}
			"MSG" -
			"JOIN" -
			"QUIT" {
				ListeParties::Protocole $opt $args
			}
			"WIN" -
			"END" -
			"HIT" -
			"PLAY" -
			"GRID" {
				JouePartie::Protocole $opt $args
			}
			"ERROR" {
				grab release
				tk_messageBox -icon error -title "$Glob(prg)" -type ok \
				    -message [lindex $args 0]
				exit
			}
		}
	}
# Quand wish n'aura rien a faire alors il ira voir si une nouvelle ligne de
# message est disponible et l'interpretera (c'est ici que tout se joue) 
	update
	after idle { TraiteProtocole [lit_socket] }
}


#############################################################################
#
# Programme principal
#
############################################################################

# Suppression de la fenetre mere de wish
wm withdraw .

# Verification et recuperation des arguments
if {$argc<1} {	bgerror "Usage: $argv0 serveur \[no_port\]" }
set host [lindex $argv 0]
set port [lindex $argv 1]
if { $port == "" } { set port 6543 }


# On charge les icones pour le jeu
InitIcones $pwd/../images

# On se connecte ensuite au serveur
if {[catch { ouvre_connexion $host $port  } err_msg]} { bgerror $err_msg }

# On affiche la fenetre de demarrage
ListeParties::Draw
set Glob(phase) ListeParties

# Et on lance la capture et analyse des messages sur la socket
TraiteProtocole ""

# Ensuite ca se debrouille tout seul
