#!/usr/local/bin/wish -f
# 
# Un ensemble de Mega-Widget avec un look potable
#
# Pierre Vassellerie - (c) Juin 1995
# Pierre.Vassellerie@obspm.fr
#


###########################################################################
# ScrolledListbox
# (listbox avec barres de defilement sans croisement)
#
# Les options sont celles de listbox
#
###########################################################################
proc ScrolledListbox { parent args } {

  frame $parent

  eval {listbox $parent.list -yscrollcommand [list $parent.sy set] \
		             -xscrollcommand [list $parent.sx set]} $args
  scrollbar $parent.sy -orient vertical -command [list $parent.list yview]

  frame $parent.bottom
    scrollbar $parent.sx -orient horizontal -command [list $parent.list xview]


    set pad [expr [$parent.sy cget -width] + 2* \
                 ([$parent.sy cget -bd] + \
		  [$parent.sy cget -highlightthickness])]
    frame $parent.pad -width $pad -height $pad

    pack $parent.pad -in $parent.bottom -side right
    pack $parent.sx  -in $parent.bottom -side bottom -fill x
  pack $parent.bottom -side bottom -fill x

  pack $parent.sy -side right -fill y
  pack $parent.list -side left -fill both -expand true

  return $parent.list
}

###########################################################################
# ScrolledText
# (text avec barres de defilement sans croisement)
#
# Les options sont celles de text
#
###########################################################################
proc ScrolledText { parent args } {

  frame $parent

  eval {text $parent.text -yscrollcommand [list $parent.sy set] \
		          -xscrollcommand [list $parent.sx set]} $args
  scrollbar $parent.sy -orient vertical -command [list $parent.text yview]

  frame $parent.bottom
    scrollbar $parent.sx -orient horizontal -command [list $parent.text xview]


    set pad [expr [$parent.sy cget -width] + 2* \
                 ([$parent.sy cget -bd] + \
		  [$parent.sy cget -highlightthickness])]
    frame $parent.pad -width $pad -height $pad

    pack $parent.pad -in $parent.bottom -side right
    pack $parent.sx  -in $parent.bottom -side bottom -fill x
  pack $parent.bottom -side bottom -fill x

  pack $parent.sy -side right -fill y
  pack $parent.text -side left -fill both -expand true

  return $parent.text
}


###########################################################################
# LabEntry
# (une entry avec son label)
#
# Les options sont :
#			-text "le texte du label"
#			-textvariable la_var_de_l_entry
#                       -lwidth lar_label
#                       -ewidth lar_entry
#
###########################################################################
proc LabEntry { parent args } {

	frame $parent
	set cmd1 [list label $parent.lab -anchor w]
	set cmd2 [list entry $parent.ent]
	for {set i 0} {$i< [llength $args]} {incr i} {
		switch -- [lindex $args $i] {
			"-text" {
				set cmd1 "$cmd1 [lrange $args $i [expr $i+1]]"
			}
			"-textvariable" {
				set cmd2 "$cmd2 [lrange $args $i [expr $i+1]]"
			}
			"-lwidth" {
				set cmd1 "$cmd1 -width [lindex $args [expr $i+1]]"
			}
			"-ewidth" {
				set cmd2 "$cmd2 -width [lindex $args [expr $i+1]]"
			}
		}
	}
	eval $cmd1
	eval $cmd2
	pack $parent.lab -side left -anchor w
	pack $parent.ent -side right -padx 5
	
	return $parent.ent

}

###########################################################################
# LabCheckbutton
# (un checkbutton avec son label a droite)
#
# Les options sont :
#			- parent
#			- txt
#			- var
#			- args
#
###########################################################################
proc LabCheckbutton { parent args } {

    frame $parent
    set cmd1 [list label $parent.lab -anchor e]
    set cmd2 [list checkbutton $parent.check -anchor w]
        for {set i 0} {$i< [llength $args]} {incr i} {
                switch -- [lindex $args $i] {
                        "-text" {
                                  set cmd1 "$cmd1 [lrange $args $i [expr $i+1]]
                                }
                        "-variable" {
                                set cmd2 "$cmd2 [lrange $args $i [expr $i+1]]"
                                }
                        }
        }
        eval $cmd1
        eval $cmd2
    
    pack $parent.lab -side left -anchor e
    pack $parent.check -side right
    
    return $parent
}

###########################################################################
# LabRadio
# (un radiobutton avec son label)
#
# Les options sont :
#			-text "le texte du label"
#			-variable la_var_du_radiobutton
#                       -width lar_label
#                       -value la_valeur_du_radiobutton
#
###########################################################################
proc LabRadio { parent args } {

	frame $parent
	set cmd1 [list label $parent.lab -anchor w]
	set cmd2 [list radiobutton $parent.radio -anchor w]
	for {set i 0} {$i< [llength $args]} {incr i} {
		switch -- [lindex $args $i] {
			"-text" {
				set cmd1 "$cmd1 [lrange $args $i [expr $i+1]]"
			}
			"-variable" {
				set cmd2 "$cmd2 [lrange $args $i [expr $i+1]]"
			}
			"-width" {
				set cmd1 "$cmd1 -width [lindex $args [expr $i+1]]"
			}
			"-value" {
				set cmd2 "$cmd2 [lrange $args $i [expr $i+1]]"
			}
		}
	}
	eval $cmd1
	eval $cmd2
	pack $parent.lab $parent.radio -side left -padx 5
	
	return $parent.radio
}


###########################################################################
# Message
# (une fenetre de message a la bonne taille avec attente)
#
# Les options sont :
#			- le titre
#			- le message a afficher
#			- l'icone Tk a afficher
#
###########################################################################
proc Message { title msg icone } {

    bell
    after idle {.dialog999.msg configure -wraplength 10c}
    set i [tk_dialog .dialog999 $title $msg $icone 0 OK]
    return $i
}