/* -*-linux-c-*-
** $Id: debug.h,v 1.1 1998/04/22 19:07:05 enlight Exp $
** $Log: debug.h,v $
** Revision 1.1  1998/04/22 19:07:05  enlight
** Initial revision
**
*/

/*   FORMAT DES MESSAGES/MACROS
**   --------------------------
**   INTERNAL_ERROR;
**	Envoie message erreur immediatement. (sans quitter).
**   NOTICE(format, args...);
**	Envoie un message moins important.
**	Exemple: NOTICE("Hello world %d\n", x);
**   ASSERT(x);
**	Verifie que x est vrai. Sinon renvoie un message d'erreur.
**	Exemple: ASSERT(toto == 3);
**   PASSERT(x, format, args...);
**	Verifie que x est vrai, sinon affiche message format.
**	Exemple: PASSERT(p->next != NULL, "toto est egal a %d\n", x);
**   DEBUGL(fmt, args...);
**	Envoie message de debug, avec num ligne plus nom prog.
**	Exemple: DEBUGL("test %d\n", x);
**
*/

#ifndef _DEBUG_H_
#define _DEBUG_H_

#include "../config.h"

#ifdef HAVE_SYSLOG
# include <syslog.h>
#else
# ifdef HAVE_SYS_SYSLOG_H
#  include <sys/syslog.h>
# else
#  ifdef HAVE_SYSLOG_H
#   include <syslog.h>
#  else
#   error Syslog isnt available ?
#  endif
# endif
#endif

#ifdef  HAVE_UNISTD_H
# include <unistd.h>
#endif

/* Constantes par defaut a changer s'il le faut */
#define DEBUG 1

#define LOGPERSO LOG_LOCAL1

#define __DEBUG_MSG(format, args...)\
	syslog(LOG_NOTICE, "%s:%i: " format "\n", \
	       __FILE__, __LINE__, ##args)

#define INTERNAL_ERROR \
	syslog(LOG_NOTICE, "%s:%i: Erreur interne.",\
		__FILE__, __LINE__)

#define NOTICE(format, args...)\
	syslog(LOG_NOTICE, format "\n", ##args)

#ifndef DEBUG
# define ASSERT(x)		do {} while(0)
# define PASSERT(x, args...)	do {} while(0)
# define DEBUGL(args...)	do {} while(0)

#else
# define ASSERT(x) \
	if (x) {} else {\
		__DEBUG_MSG("assertion %s a echouee.", #x); }
# define PASSERT(x, args...)\
	if (x) {} else {\
		__DEBUG_MSG("assertion %s a echouee.", #x); \
		__DEBUG_MSG(##args); }
#define DEBUGL(args...) __DEBUG_MSG(##args)
#endif

#endif
