/* -*-linux-c-*-
** $Id: libconnex.h,v 1.2 1998/04/23 03:26:22 enlight Exp $
** $Log: libconnex.h,v $
** Revision 1.2  1998/04/23 03:26:22  enlight
** Retire les variable inutilisee.
**
** Revision 1.1  1998/04/22 19:06:55  enlight
** Initial revision
**
*/

#ifndef _LIBCONNEX_H_
#define _LIBCONNEX_H_

#include "servMorp.h"

#ifdef HAVE_SYS_SOCKET_H
# include <sys/socket.h>
#endif
#ifdef HAVE_NETINET_IN_H
# include <netinet/in.h>
#endif
#ifdef HAVE_ARPA_INET_H
# include <arpa/inet.h>
#endif

#include <netdb.h>

#ifdef HAVE_SIGNAL_H
# include <signal.h>
#else
# ifdef HAVE_SYS_SIGNAL_H
#  include <sys/signal.h>
# endif
#endif

#ifdef HAVE_SELECT_H
# include <select.h>
#else
# ifdef HAVE_SYS_SELECT_H
#  include <sys/select.h>
# else
#  ifdef HAVE_LINUX_TIME_H
#   include <linux/time.h>
#  else
#   ifdef HAVE_TIME_H
#    include <time.h>
#   endif
#  endif
#  include <time.h>
# endif
#endif
#ifdef HAVE_SYS_WAIT_H
# include <sys/wait.h>
#endif

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif
#include <sys/stat.h>
#include <fcntl.h>

#ifdef NOFILE
# define SOCKMAX_FD NOFILE - 3
#else
#  define SOCKMAX_FD 256
#endif

#if 0

#ifdef _POSIX_OPEN_MAX
# define SOCKMAX_FD _POSIX_OPEN_MAX
#else
# ifdef _POSIX_FD_SETSIZE
#  define SOCKMAX_FD _POSIX_FD_SETSIZE
# else
#  ifdef __FD_SETSIZE
#   define SOCKMAX_FD __FD_SETSIZE
#  else
#   define SOCKMAX_FD 256
#  endif
# endif
#endif

#endif

#ifndef max
# define max(a, b) (((a) > (b)) ? (a) : (b))
#endif

enum {
   EVENT_NONE,
   EVENT_LOSTCONNEX,
   EVENT_ERROR,
   EVENT_EWOULDBLOCK,
   EVENT_HANG,
   EVENT_REPLY,
   EVENT_NEWCONNEX,
   EVENT_INTERRUPT,
   EVENT_BADDESCRI
} SockEventType;


/* Definition d'un Joueur */
typedef struct PlayerSock_ {
    char playerName[MAXPLAYERNAME + 1]; /* Player's name */
    int nbGamesPlayed;          /* nb of currently played games */
    int vide; 			/* check to see if this cell is free */
    char machineName[MAXMACHINENAME + 1];
    struct sockaddr_in addr;  	/* socket structure + infos */
} PlayerSock;

/* Definition d'une partie */
typedef struct Game_ {
    int type;                   /* type de la partie */
    long idGame;                /* Games's Id */
    int width;                  /* nb of cells horizontally */
    int height;                 /* nb of cells vertically */
    int *grid;                  /* grid contents */
    int nbPlayers;              /* nb of players currently in that game */
    int nbHit;                  /* nb of HIT in that game */
    char *icon;			/* Liste des playerId disponibles */
    char gameName[MAXGAMENAME + 1];
} Game;

typedef struct ListPlayerSockItem_ {
    int numSock;
    int playerId;
    struct ListPlayerSockItem_ *next;
} ListPlayerSockItem;

typedef struct ListGameItem_ {
    Game *game;
    struct ListGameItem_ *next;
    ListPlayerSockItem *head;
} ListGameItem;


typedef struct ListVote_ {
    int hasVoted;
    int numGameToVoteOn;
    int numPlayerToVoteFor;          /* indice of player which want to JOIN a game */
    int nbPlayersAtVote;
    struct ListVote_ *next;
} ListVote;

typedef struct Games_ {
    char *listIdGame;                /* Nb total d'id de partie deja alloue */
    ListVote *listVotes;             /**/
    ListGameItem *listGames;         /**/
    PlayerSock players[SOCKMAX_FD];  /**/
} Games;

typedef struct Sockets_ {
    /* Not yet used */
    int (*fctNewClientConnex)(struct Sockets_ *s);
    int (*fctProcessMessage)(struct Sockets_ *s, int numSock);

    int rFirstFd;                   /* numero du premier descripteur read */
    int newConnexFd;                /* numero de la socket gerant les demandes
				       de nouvelles connexions */
    int width;                      /* le num du plus haut desc */
    fd_set coreRfds;                /* ensemble des descripteurs en lecture */
    Games games;
} Sockets;

typedef struct SockEvent_ {
    int type;
    int fd;
    char content[MAXLINELENGTH];
    int contentLen;
    Sockets *ptrSockets;
    PlayerSock *socket;
} SockEvent;

/* Declaration des prototypes */
int socket_print(int sock, char *msg);
int socket_initDaemon(int doNotDetach);
SockEvent *socket_waitEvent(Sockets *sockets);
SockEvent *socket_newClientConnex(Sockets *sockets);
SockEvent *socket_processClientMessage(Sockets *sockets, int numSock);
void socket_setup(Sockets *sockets, int port);
void socket_detachId(void);

#endif





