/* -*-linux-c-*-
** $Id: libconnex.c,v 1.3 1998/04/24 18:26:57 enlight Exp enlight $
** $Log: libconnex.c,v $
** Revision 1.3  1998/04/24 18:26:57  enlight
** Modification de la fonction socket_print pour qu'elle supporte l'envoi
** 	des messages tenant dans plusieurs paquets.
**
** Revision 1.2  1998/04/23 03:25:28  enlight
** Ajout dans socket_close de la reinitialisation des variable concernant le joueur a retirer.
** Remplace les exit(0) et exit(1) par des exit(EXIT_FAILURE) et exit(EXIT_SUCCESS)
** Remplace les exit() de type catastrophes pas des kill(getpid(), SIGTERM)
**
** Revision 1.1  1998/04/22 19:05:54  enlight
** Initial revision
**
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <errno.h>
#include "servMorp.h"
#include "libconnex.h"

extern int errno;
extern int h_errno;

void term_handler(int numsig)
{
    int fd = 0;
    
    DEBUGL("On arrete le serveur", ".");
    
    while(fd <= SOCKMAX_FD) {
	close(fd);
	fd++;
    }
    exit(EXIT_SUCCESS);
}

void chld_handler(int numsig)
{
    int status;
    
#ifdef HAVE_WAIT3
    wait3(&status, WNOHANG, NULL);
#elif defined (WNOHANG)
    waitpid(-1, &status, WNOHANG);
#else
    wait(&status);
#endif
}

void socket_setup(Sockets *sockets, int port)
{
    int sockFd;
    struct sockaddr_in myAddr;
    /* Pour activer le SO_REUSEADDR */
    int flag = 1;

    /* Initialisation des sockets */
    if ((sockFd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
        DEBUGL("Pas pu creer la socket.", ".");
        kill(getpid(), SIGTERM);
    }
    
    /* 
    ** Stockage du numero de la socket qui gere les nouvelles connexions
    ** pour utilisation ulterieure, au cas ou le numero n'est pas 3 (ce qui
    ** en theorie devrait etre le cas...;-)
    */ 
    sockets->newConnexFd = sockFd;
    
    /* Retirer le MSL pour pouvoir reutiliser le port immediatement */
    if (setsockopt(sockFd, SOL_SOCKET, SO_REUSEADDR, &flag, 
                   sizeof(int)) == -1) {
    	DEBUGL("Pas pu activer SO_REUSEADDR.", ".");
        kill(getpid(), SIGTERM);
    }
    
    myAddr.sin_family = AF_INET;
    if (port == 0)
       port = MONPORT;
    myAddr.sin_port = htons(port);
    myAddr.sin_addr.s_addr = INADDR_ANY;	/* rempli avec mon adr IP */
    bzero((char*)&(myAddr.sin_zero), 8);
    
    if (bind(sockFd, (struct sockaddr *)&myAddr, 
             sizeof(struct sockaddr)) == -1) {
    	DEBUGL("Pas pu binder la socket.", ".");
        kill(getpid(), SIGTERM);
    }
    if (listen(sockFd, 10) == -1) {
	DEBUGL("Listen a echoue.", ".");
        kill(getpid(), SIGTERM);
    }
    
    /* Initialisation de la structure */
    FD_ZERO(&(sockets->coreRfds));

    /* On ajoute la socket d'ecoute des nouvelles connexions */
    FD_SET(sockFd, &(sockets->coreRfds));
    
    /*
    ** Les 4 premiers desc. sont alloues pour stdin, stdout, stderr, syslog.
    ** Le cinquieme est reserve pour l'ecoute des nouvelles connex.
    */
    sockets->rFirstFd = sockets->newConnexFd;
    sockets->width = sockets->rFirstFd;
}


void socket_detachId(void)
{
    int fd;

    fd = -1;

#ifdef HAVE_SETSID
    /* Pour les System V */
    if (setsid() == -1) {
        perror("Pas pu executer setsid()");
        exit(EXIT_FAILURE);
    }
#else
# ifdef BSD
    /* On change le groupe */
    if (setpgrp(0, getpid()) == -1) {
        perror("Pas pu executer setpgrp()");
        exit(EXIT_FAILURE);
    }
    fd = open("/dev/tty", O_RDWR);
    /* On lache le terminal */
    if (fd != -1) {
        ioctl(fd, TIOCNOTTY, 0);
        close(fd);
    }
# else
#  ifdef SVR4
    if (setpgrp() == -1) {
        perror("Pas pu executer setpgrp()");
        exit(EXIT_FAILURE);
    }
#  endif
# endif
#endif
}

int socket_initDaemon(int doNotDetach)
{
    int fd = -1;


    if (!doNotDetach){
        /*
        ** Dedoublement du process, avec suppression du pere
        */
        switch(fork()){
        case -1:
            perror("fork()");
            exit(EXIT_FAILURE);
        case 0:       /* process fils */
            break;
        default:      /* process pere se termine*/
            exit(EXIT_SUCCESS);
        }
        
        /*
        ** On se dissocie du processus de groupe, et du terminal
        */
        socket_detachId();
        
        /*
        ** On reeffectue un fork pour empecher la possibilite de reacquisition
        ** par un terminal.
        */
        if (fork() != 0)
            exit(EXIT_SUCCESS);  /* Le pere disparait */
        
        /* 
        ** On ferme TOUS les descripteurs disponibles Ainsi que STDIN, 
        ** STDOUT, STDERR
        */
        for (fd = 0; fd < SOCKMAX_FD; fd++)
            close(fd);
    }
    
    chdir("/");
    umask(0);

    /* 
    ** Il semble qu'il y ait des problemes pour certains 
    ** librairies qui veulent stdin, stdout, stderr
    */
    fd = open("/dev/null", O_RDWR); /* stdin */
    (void) dup(fd); /* stdout */
    (void) dup(fd); /* stderr */

    /* Redirection des signaux pour traiter les sorties */
    signal(SIGTERM, term_handler);
    signal(SIGINT, term_handler);
    signal(SIGHUP, term_handler);
    signal(SIGQUIT, term_handler);
    signal(SIGCHLD, chld_handler);
    signal(SIGTTOU, SIG_IGN);
    signal(SIGTTIN, SIG_IGN);
    signal(SIGTSTP, SIG_IGN);
    signal(SIGPIPE, SIG_IGN);

    /*
    ** Ouverture / Initialisation de syslog
    */
#ifdef HAVE_SYSLOG
    openlog("", LOG_PID|LOG_NDELAY, LOGPERSO);
#endif
    NOTICE("Initialisation du serveur", ".");

    return 0;
}


int socket_print(int sock, char *msg)
{
    int bytesSent = 0, toSend, currentSent;
    char *buf = msg;

    toSend = strlen(msg);
    
    while(bytesSent < toSend){
        do {
            currentSent = send(sock, buf, toSend - bytesSent, 0);
        } while ( (currentSent < 0) && (errno == EINTR));
        if (currentSent <= 0)
            return currentSent;
        bytesSent += currentSent;
        buf += currentSent;
    }

    return 0;
}

SockEvent *socket_newClientConnex(Sockets *sockets)
{
    int newSock, sin_size = sizeof(struct sockaddr_in);
    struct sockaddr_in theirAddr;
    struct hostent *he;
    SockEvent *event;


    event = (SockEvent *)malloc(sizeof(SockEvent)* 1);

    /* We memorize the main structure */
    event->ptrSockets = sockets;

    if ((newSock = accept(sockets->newConnexFd, 
                          (struct sockaddr *)&theirAddr, 
                          &sin_size)) == -1) {
        DEBUGL("accept a echoue sur accept %d\n", sockets->newConnexFd);
        event->type = EVENT_ERROR;
        return event;
    }

    /* We get the real DNS Name of our player */
    he = gethostbyaddr((char *)&(theirAddr.sin_addr), 
                              sizeof(struct in_addr), 
                              AF_INET);
    if (he == NULL)
        strncpy(sockets->games.players[newSock].machineName, 
                inet_ntoa(theirAddr.sin_addr),
                MAXMACHINENAME);
    else
        strncpy(sockets->games.players[newSock].machineName,
                he->h_name,
                MAXMACHINENAME);

    NOTICE("Nouvelle connexion(%d) par %s\n", 
           newSock, 
           sockets->games.players[newSock].machineName);

    if (sockets->games.players[newSock].vide){ /* C'est libre ? */
        /* On ajoute la socket a la liste des autres */
        FD_SET(newSock, &sockets->coreRfds);

        /* Reservation de la cellule */
        sockets->games.players[newSock].vide = 0;

        memcpy(&sockets->games.players[newSock].addr, &theirAddr, 
               sizeof(struct sockaddr_in));

        /* Determination et Mise a jour du num du plus grand descripteur */
        sockets->width = max(sockets->width, newSock);

        event->type = EVENT_NEWCONNEX;
	event->fd = newSock;

        return event;
        
    } else {
        DEBUGL("FATAL Initialisation d'une cellule deja utilisee (%d)!", newSock);
        kill(getpid(), SIGTERM);
    }


    event->type = EVENT_NONE;
    return event;
}

void socket_close(Sockets *sockets, int numSock)
{
    NOTICE("Fermeture de la socket (%d)", numSock);
    close(numSock);
    FD_CLR(numSock, &sockets->coreRfds);

    /* Recalcul du plus grand descripteur */
    if (numSock == sockets->width)
        sockets->width--;
}

SockEvent *socket_processClientMessage(Sockets *sockets, int numSock)
{
    int nbCarRead;
    SockEvent *event;

    event = (SockEvent *)malloc(sizeof(SockEvent)* 1);

    /* We memorize the main structure */
    event->ptrSockets = sockets;

    ASSERT(numSock < SOCKMAX_FD);
    PASSERT(sockets->games.players[numSock].vide != -1, 
            "Erreur, le fd recupere (%d) pointe sur une entree non initialisee\n",  numSock);

    nbCarRead = read(numSock, event->content, MAXLINELENGTH);
    if (nbCarRead <= 0){
        DEBUGL("Erreur Lecture socket %d\n", numSock);
        socket_close(sockets, numSock);

        event->fd = numSock;
        event->type = EVENT_LOSTCONNEX;
        return event;
    }

    event->content[nbCarRead-1] = '\0';
    event->contentLen = nbCarRead;
    
    NOTICE("Recu message de (%d): [%s]", numSock, event->content);

    event->fd = numSock;
    event->type = EVENT_REPLY;
    return event;
}

SockEvent *socket_waitEvent(Sockets *sockets)
{
    int iterFd, nFds;
    fd_set rfds;
    SockEvent *event;

    if (sockets == NULL)
        return NULL;


    event = (SockEvent *)malloc(sizeof(SockEvent)* 1);
    /* We memorize the main structure */
    event->ptrSockets = sockets;

    /* On travaille sur la copie */
    memcpy(&rfds, &(sockets->coreRfds), sizeof(sockets->coreRfds));

    /*
    ** On attend un evenement ou une connexion sur un des descripteurs.
    ** Le TIMEOUT est mis a +infini
    */
#ifdef LINUX
    nFds = select(sockets->width + 1, &rfds, NULL, NULL, NULL);
#else
    nFds = select(sockets->width + 1, (int *)&rfds, NULL, NULL, NULL);
#endif
    if (nFds == -1) {
        switch(errno){
        case EINTR:
            event->type = EVENT_INTERRUPT;
        case EBADF:
            event->type = EVENT_BADDESCRI;
        default:
            event->type = EVENT_ERROR;
        }
        return event;
    } else if (nFds == 0){
        event->type = EVENT_NONE;
        return event;
    }

    /* No need to keep the event structure now, as it will be recreated
       into the following functions */
    ASSERT(event != NULL);
    free(event);

    /* C'est la socket speciale qui se charge des nouvelles connexions */
    if (FD_ISSET(sockets->newConnexFd,&rfds)){
        return (socket_newClientConnex(sockets));
    }
    
    for (iterFd = sockets->rFirstFd; iterFd <= sockets->width; iterFd++){
        if (FD_ISSET(iterFd, &rfds) && FD_ISSET(iterFd, &sockets->coreRfds)){
            NOTICE("Socket %d declenchee !\n", iterFd);
            return (socket_processClientMessage(sockets, iterFd));
        }
    }

    return NULL;
}
