/* -*-linux-c-*-
** $Id: libmorpion.c,v 1.2 1998/04/23 03:27:25 enlight Exp $
** $Log: libmorpion.c,v $
** Revision 1.2  1998/04/23 03:27:25  enlight
** Ajout d'une fonction morpion_init.
** Commence a remplacer les MORPION_* qui sont OK par MORPION_OK.
**
** Revision 1.1  1998/04/22 19:04:46  enlight
** Initial revision
**
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "servMorp.h"
#include "libconnex.h"
#include "libutils.h"
#include "global.h"

#include "libmorpion.h"

/* Identification DES modes de parties !! */
char* partie[] = {"Morpion", "Puissance4"};

INLINE
void morpion_playerReInit(int userNum, Sockets *sockets)
{

    sockets->games.players[userNum].vide = 1;
    sockets->games.players[userNum].playerName[0] = '\0';
    sockets->games.players[userNum].nbGamesPlayed = 0;
    sockets->games.players[userNum].machineName[0] = '\0';
}

/* 
** Function: morpion_init
**       In: the main structure containing the Sockets structure.
**   Return: nothing
** Comments: This function initializes the different structures
**           used in the project.
*/
valMorpion morpion_init(Sockets *sockets)
{
	int i;
	/*
	** Remise a zero (plutot -1) de la table statique des joueurs
	** pour connaitre les cellules non initialisee/utilisees
	*/
	for (i = 0; i < SOCKMAX_FD; i++)
		morpion_playerReInit(i, sockets);

	sockets->games.listVotes = NULL;
	sockets->games.listGames = NULL;
	if ((sockets->games.listIdGame = (char*)malloc(sizeof(char) *
	    (LISTIDGAME / OCTET + 1))) == NULL){
		NOTICE("Not enought memory !!", ".");
		return MORPION_ERROR;
	}
	utils_initBit(&(sockets->games.listIdGame), LISTIDGAME);

	return MORPION_OK;
}

/* 
** Function: morpion_existGameNum
**       In: the game's Id
**           the main structure containing the Game structure.
**   Return: ListGameItem* on the game if it exist
**           NULL if the game don't exist
** Comments: This function test if a game already exist and return
**           it.
*/
ListGameItem* morpion_existGameNum(long idGame, Games *games)
{
	ListGameItem *listGameItemIt = NULL;
	
	if (!games)
		return NULL;
	
	for (listGameItemIt = games->listGames ; listGameItemIt ; 
			listGameItemIt = listGameItemIt->next)
		if (listGameItemIt->game->idGame == idGame)
			return listGameItemIt;
	return NULL;	
}

/* 
** Function: morpion_existGame
**       In: the game's name
**           the main structure containing the Game structure.
**   Return: MORPION_OK if the game exist
**           MORPION_GAMENOEXIST if the game don't exist
**           MORPION_ERROR if the game structure is NULL
** Comments: This function test if a game already exist.
*/
valMorpion morpion_existGame(char *gameName, Games *games)
{
	ListGameItem *listGameItemIt = NULL;
	
	if (!gameName || !games)
		return MORPION_ERROR;
	
	for (listGameItemIt = games->listGames ; listGameItemIt ; 
			listGameItemIt = listGameItemIt->next)
		if (!strcasecmp(listGameItemIt->game->gameName, gameName))
			return MORPION_OK;
	return MORPION_GAMENOEXIST;	
}

/* 
** Function: morpion_existPlayer
**       In: the player's name
**           the main structure containing the Game structure.
**   Return: Socket number if the player exist
**           -1 the player don't exist in that game or if the 
**           Game structure is NULL
** Comments: This function test if a player already exist.
*/
int morpion_existPlayer(char *playerName, Games *games)
{
	int bcl = 0;
	
	if (!games || !playerName)
		return -1;

	for (bcl = 0 ; bcl < SOCKMAX_FD ; bcl++)
		if (!games->players[bcl].vide && 
		    !strcasecmp(games->players[bcl].playerName, playerName))
			return bcl;

	return -1;
}

/* 
** Function: morpion_noticeToAllPlayers
**       In: the idGame (-1 for all)
**           the socket number (num > -1 => broadcast on all
**           without num, num = -1 => broadcast on all)
**           the message to broadcast
**           the list of destinataire (MORPION_GAME for player
**           in the current game, MORPION_ALLGAME for players
**           in all the games)
**           broadcast even or no on the current player 
**           MORPION_WITHME or MORPION_WITHOUTME
**           the main structure containing the Game structure.
**   Return: MORPION_OK if the message has been send
**           MORPION_GAMENOEXIST if the idGame don't exist
**           MORPION_ERROR if the Game structure or the 
**           message are NULL, or if an allocation error occurs
** Comments: This function test if a player already exist.
*/
valMorpion morpion_noticeToAllPlayers(int numSock, long idGame, char dest,
           char *message, Games *games)
{
	ListGameItem *listGameItem = NULL;
	ListPlayerSockItem *listPlayerSockItem = NULL;
	int bcl, alloc;
	char *bits = NULL;

	if (!message || !games)
		return MORPION_ERROR;

	/*
	** Verification de la partie dans le cas d'un BROADCAST_GAME
	*/
	if ((dest & BROADCAST_GAME) == BROADCAST_GAME){
		if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL)
			return MORPION_GAMENOEXIST;
	}

	/*
	** Verification du joueur dans le cas d'un BROADCAST_PLAYER
	*/
	if ((dest & BROADCAST_PLAYER) == BROADCAST_PLAYER){
		if (games->players[numSock].vide)
			return MORPION_PLAYERNOEXIST;

		if ((dest & BROADCAST_GAME) == BROADCAST_GAME){
			listPlayerSockItem = listGameItem->head;
			for (bcl = 0 ; bcl < listGameItem->game->nbPlayers ; bcl++){
				if (listPlayerSockItem->numSock == numSock)
					break;
				listPlayerSockItem = listPlayerSockItem->next;
			}
			if (!listPlayerSockItem || listPlayerSockItem->numSock != numSock){
				return MORPION_PLAYERNOEXIST;
			}
		} else if ((dest & BROADCAST_ALLGAME) == BROADCAST_ALLGAME){
			if (games->players[numSock].nbGamesPlayed == 0){
				return MORPION_PLAYERNOEXIST;
			}
		}
	}

	/*
	** On broadcast le message a toutes les parties
	*/
	if ((dest & BROADCAST_ALLGAME) == BROADCAST_ALLGAME){
		alloc = SOCKMAX_FD / OCTET + 1;
		if ((bits = (char*) malloc(sizeof(char) * alloc)) == NULL){
			NOTICE("Not enought memory",".");
			return MORPION_ERROR;
		}

		utils_initBit(&bits, SOCKMAX_FD);

		for (listGameItem = games->listGames ; listGameItem ;
			   listGameItem = listGameItem->next){
			listPlayerSockItem = listGameItem->head;
			for (bcl = 0 ; bcl < listGameItem->game->nbPlayers ; bcl++){
				/*
			** si on ne broadcast pas le player (par defaut) en
			** cours et que nous sommes dessus, on passe au 
			** player suivant
			*/
				if(((dest & BROADCAST_PLAYER) != BROADCAST_PLAYER) &&
				   (listPlayerSockItem->numSock == numSock))
					continue;
				
				if (utils_isBitSet(listPlayerSockItem->numSock, bits, SOCKMAX_FD)
				    == UTILS_UNSET){
					socket_print(listPlayerSockItem->numSock, message);
					utils_setBit(listPlayerSockItem->numSock, &bits, SOCKMAX_FD);
				}	
				listPlayerSockItem = listPlayerSockItem->next;
			}
		}
	}
	/*
	** On broadcast le message a la partie idGame
	*/
	else if ((dest & BROADCAST_GAME) == BROADCAST_GAME){
		listPlayerSockItem = listGameItem->head;
		for (bcl = 0 ; bcl < listGameItem->game->nbPlayers ; bcl++){
			/*
		** si on ne broadcast pas le player (par defaut) en
		** cours et que nous sommes dessus, on passe au 
		** player suivant
		*/
			if(((dest & BROADCAST_PLAYER) != BROADCAST_PLAYER) &&
				   (listPlayerSockItem->numSock == numSock))
					continue;

			socket_print(listPlayerSockItem->numSock, message);
			listPlayerSockItem = listPlayerSockItem->next;
		}
	}
	/*
	** On broadcast le message a tous les connectes
	*/
	else if ((dest & BROADCAST_ALLPLAYERS) == BROADCAST_ALLPLAYERS){
		for (bcl = 0 ; bcl < SOCKMAX_FD ; bcl++){
			if(((dest & BROADCAST_PLAYER) != BROADCAST_PLAYER) && (bcl == numSock))
				continue;

			if (games->players[bcl].vide != 1)
			socket_print(bcl, message);
		}
	}

	return MORPION_OK;
}

/* 
** Function: morpion_askVote
**       In: the indice in array of players ask for JOIN
**           the idGame
**           the main structure containing the Game structure.
**   Return: MORPION_OK if vote has been created
**           MORPION_GAMENOEXIST if the game not exist
**           MORPION_PLAYERNOEXIST if the player not exist
**           MORPION_ERROR if an alocation error occur
** Comments: This function create a vote for a player in a game.
*/
valMorpion morpion_askVote(int numSock, int idGame, Games *games)
{
	ListGameItem *listGameItem = NULL;
	ListVote *newListVote = NULL;
	
	/*
	** Verification de l'existence de la partie et du joueur
	*/
	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		return  MORPION_GAMENOEXIST;
	}
	if (games->players[numSock].vide){
		NOTICE("Player (%d) not exist -> no vote", numSock);
		return  MORPION_PLAYERNOEXIST;
	}
	
	if ((newListVote = (ListVote*)malloc(sizeof(ListVote))) == NULL){
		NOTICE("Not enought memory for vote for player (%d)", numSock);
		return MORPION_ERROR;
	}

	newListVote->numPlayerToVoteFor = numSock;
	newListVote->numGameToVoteOn = idGame;
	NOTICE("nbplayers=%d", listGameItem->game->nbPlayers);
	newListVote->nbPlayersAtVote = listGameItem->game->nbPlayers;
	newListVote->hasVoted = 0; /* nb of players that have already voted */

	/*
	** Insersion de la cellule en tete de liste
	*/
	newListVote->next = games->listVotes;
	games->listVotes = newListVote;

	NOTICE("Vote created for player %s(%d) in game %s(%d)",
	       games->players[numSock].playerName, numSock,
			 listGameItem->game->gameName, idGame);
	return MORPION_OK;	
}

/* PARAMS:
   numPartie = partie d'ou a ete appele la fonction
 */
valMorpion morpion_removeVote(int numPlayerToAccept, int numGameToVoteOn,
                        Games *games)
{
    ListVote *ptrVote = games->listVotes;
    ListVote *ptrVoteDel = games->listVotes;

    /* Looking for the good cell */
    while (ptrVoteDel != NULL){
        if ((ptrVoteDel->numPlayerToVoteFor == numPlayerToAccept) &&
            (ptrVoteDel->numGameToVoteOn == numGameToVoteOn)){
            /* We found the good cell */
				
            /* On est en tete de liste */
            if (ptrVoteDel == games->listVotes){
                games->listVotes = ptrVoteDel->next;
                free(ptrVoteDel);
                return MORPION_OK;
            }

            ptrVote->next = ptrVoteDel->next;
            free(ptrVoteDel);
				return MORPION_OK;
        }
        ptrVote = ptrVoteDel;
        ptrVoteDel = ptrVoteDel->next;
    }

    return MORPION_VOTENOEXIST;
}


/* PARAMS:
   numPartie = partie d'ou a ete appele la fonction
 */
valMorpion morpion_vote(int numPlayerToAccept, int numGameToVoteOn,
                        Games *games)
{
    ListGameItem *listGameItem = NULL;
    ListVote *ptrVote = games->listVotes;
    
    /* Looking for the good cell */
    while (ptrVote != NULL){
        if ((ptrVote->numPlayerToVoteFor == numPlayerToAccept) &&
            (ptrVote->numGameToVoteOn == numGameToVoteOn)){
            /* We found the good cell */
            ptrVote->hasVoted++;
            listGameItem = morpion_existGameNum(numGameToVoteOn, games);

            /* Everybody has voted ? */
            if (ptrVote->hasVoted >= ptrVote->nbPlayersAtVote)
                return MORPION_ACCEPTED;
        }
        ptrVote = ptrVote->next;
    }
    
    return MORPION_OK;
}

/* 
** Function: morpion_getPlayerId
**       In: playerId (set in the function)
**           socket number
**           game's Id
**           the main structure containing the Game structure.
**   Return: MORPION_OK if the player has been found
**           MORPION_GAMENOEXIST if the game not exist
**           MORPION_PLAYERNOEXIST if the player not exist
**           set the playerId variable with the player's Id
** Comments: This function add a game to the game's list.
*/
valMorpion morpion_getPlayerId(int *playerId, int numSock, long idGame,
                               Games *games)
{
	ListGameItem *listGameItem = NULL;
	ListPlayerSockItem *listPlayerSockItem = NULL;
	int bcl;

	*playerId = - 1;
	/* Recherche d'une partie portant le meme nom */
	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		NOTICE("Game %d not exist", idGame);
		return MORPION_GAMENOEXIST;
	}

	/* verification de l'existance du joueur */
	if (games->players[numSock].vide){
		NOTICE("Player %d not exist in game %s(%d)", numSock, 
		       listGameItem->game->gameName, idGame);
		return MORPION_PLAYERNOEXIST;
	}

	listPlayerSockItem = listGameItem->head;
	for (bcl = 0 ; bcl < listGameItem->game->nbPlayers ; bcl++){
		if (listPlayerSockItem->numSock == numSock){
			*playerId = listPlayerSockItem->playerId;
			return MORPION_OK;
		}
		listPlayerSockItem = listPlayerSockItem->next;
	}
	
	return MORPION_PLAYERNOEXIST;
}

/* 
** Function: morpion_getNumSock
**       In: playerId
**           socket number (set in the function)
**           game's Id
**           the main structure containing the Game structure.
**   Return: MORPION_OK if the player has been found
**           MORPION_GAMENOEXIST if the game not exist
**           MORPION_PLAYERNOEXIST if the player not exist
**           set the numSock variable with the socket number
** Comments: This function add a game to the game's list.
*/
valMorpion morpion_getNumSock(int playerId, int *numSock, long idGame,
                               Games *games)
{
	ListGameItem *listGameItem = NULL;
	ListPlayerSockItem *listPlayerSockItem = NULL;
	int bcl;

	*numSock = -1;
	/* Recherche d'une partie portant le meme nom */
	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		NOTICE("Game %d not exist", idGame);
		return MORPION_GAMENOEXIST;
	}

	listPlayerSockItem = listGameItem->head;
	for (bcl = 0 ; bcl < listGameItem->game->nbPlayers ; bcl++){
		if (listPlayerSockItem->playerId == playerId){
			*numSock = listPlayerSockItem->numSock;
			return MORPION_OK;
		}
		listPlayerSockItem = listPlayerSockItem->next;
	}
	
	return MORPION_PLAYERNOEXIST;
}

/* 
** Function: morpion_newGame
**       In: width of the game
**           height of the game
**           game's name
**           game's Id
**           the main structure containing the Game structure.
**   Return: MORPION_OK if the game has been added
**           MORPION_GAMEEXIST if the game already exist
**           MORPION_ERROR if an alocation error occur
**           MORPION_GAMEFULL if all the game have been alocated
**           MORPION_UNKNOWGAME if the type of game is unknonw
**           return the game's id in the fourth paramater
** Comments: This function add a game to the game's list.
*/
valMorpion morpion_newGame(int width, int height, char *gameName,
			   long *idGame, int type, Games *games)
{
	int bcl;
	Game *game = NULL;
	ListGameItem *newListGameItem = NULL;

	/* Recherche d'une partie portant le meme nom */
	if (morpion_existGame(gameName, games) == MORPION_OK){
		NOTICE("Game %s already exist", gameName);
		return MORPION_GAMEEXIST;
	}
    
	/* Initialisation d'une nouvelle partie */
	if ((game = (Game*) malloc(sizeof(Game))) == NULL){
		NOTICE("Not enought memory !!",".");
		return MORPION_ERROR;
	}

	game->width = width;
	game->height = height;
	game->nbPlayers = 0;
	game->nbHit = 0;

	/* Verification du type de partie !! */
	if (type > (strlen((char*)partie) + 1)){
		NOTICE("Type de partie inexistant (%d)", type);
		return MORPION_UNKNOWGAME;
	}
	game->type = type;
	
	/* Allocation du premier Id de partie disponible ! */
	if ((game->idGame = utils_firstBitUnset(games->listIdGame, LISTIDGAME))
	     == -1){
		NOTICE("Plus de creation de partie possible !!", ".");
		free(game);
		return MORPION_GAMEFULL;
	}
	utils_setBit(game->idGame, &(games->listIdGame), LISTIDGAME);
	
	/* Allocation de la grille du jeu */
	if ((game->grid = (int*) malloc(sizeof(int) * (width * height))) == NULL){
		free(game);
		return MORPION_ERROR;
	}
	/* Initialisation de la grille a -1 */
	for (bcl = 0 ; bcl < (width * height) ; bcl++)
		game->grid[bcl] = -1;

	/* Allocation du tableau de bits pour les playerId */
	if ((game->icon = (char*)malloc(sizeof(char) * (MAXICONNUMBER / OCTET + 1)))
	     == NULL){
		NOTICE("Not enought memory", ".");
		return MORPION_ERROR;
	}
	utils_initBit(&(game->icon), MAXICONNUMBER);
	/*
	** Le bit 0 est reserve pour les joueurs qui quittent la
	** partie
	*/
	utils_setBit(0, &(game->icon), MAXICONNUMBER);

	/* Init du nim de la partie */
	strncpy(game->gameName,gameName,MAXGAMENAME);

	/* Ajout de game a la liste des parties */
	if ((newListGameItem = (ListGameItem*) malloc (sizeof(ListGameItem))) ==
		 NULL){
		free(game->grid);
		free(game);
		return MORPION_ERROR;
	}

	newListGameItem->game = game;
	newListGameItem->head = NULL;
	newListGameItem->next = games->listGames;
	games->listGames = newListGameItem;
	*idGame = game->idGame;

	return MORPION_OK;
}

/* 
** Function: morpion_deleteGame
**       In: game's Id
**           the main structure containing the Game structure.
**   Return: MORPION_OK if the game has been deleted
**           MORPION_GAMENOEXIST if the game don't exist
** Comments: This function delete a game from the game's list.
*/
valMorpion morpion_deleteGame(long idGame ,Games *games)
{
	ListGameItem *listGameItem = NULL, *listGameItemDelete = NULL;
	ListPlayerSockItem *listPlayerSockItemDelete = NULL;
    
	/*
	** Recherche de la partie a supprimer dans la liste 
	** des parties
	*/
	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		NOTICE("Game (%d) not exist -> not deleted", idGame);
		return  MORPION_GAMENOEXIST;
	}
    
	/*
	** Destruction des joueurs dans la liste de la partie
	*/
	while (listGameItem->game->nbPlayers){
		listPlayerSockItemDelete = listGameItem->head->next;
		listGameItem->head->next = listPlayerSockItemDelete->next;
		games->players[listPlayerSockItemDelete->numSock].nbGamesPlayed--;
		listGameItem->game->nbPlayers--;
		NOTICE("Player %s(%d) has been deleted from game %s(%d)",
		       games->players[listPlayerSockItemDelete->numSock].playerName,
		       listPlayerSockItemDelete->numSock, listGameItem->game->gameName,
		       idGame);
		free(listPlayerSockItemDelete);		
	}
	listGameItem->head = NULL;
	
	/*
	** Destruction de la partie.
	** Si la partie est en tete de liste
	*/
	listGameItemDelete = listGameItem;
	if (listGameItemDelete == games->listGames)
		games->listGames = listGameItemDelete->next;
	else {
		for (listGameItem = games->listGames ; listGameItem->next !=
				listGameItemDelete ; listGameItem = listGameItem->next);

		if (games->listGames == listGameItemDelete->next)
			listGameItem->next = NULL;
		else
			listGameItem->next = listGameItemDelete->next;
	}
	NOTICE("Game %s(%d) deleted", listGameItemDelete->game->gameName, idGame);
	utils_unsetBit(listGameItemDelete->game->idGame, &(games->listIdGame),
	               LISTIDGAME);
	free(listGameItemDelete->game->grid);
	free(listGameItemDelete->game->icon);
	free(listGameItemDelete->game);
	free(listGameItemDelete);
	return MORPION_OK;	
}

/* 
** Function: morpion_addPlayer
**       In: indice of the player in the players array
**           game's Id
**           the main structure containing the Game structure.
**   Return: MORPION_OK if the game has been added
**           MORPION_PLAYERINGAME if the player is already in that game
**           MORPION_GAMENOEXIST if the game don't exist
**           MORPION_NOICON if the game is full of players
** Comments: This function add a player to the player's list of the game.
*/
valMorpion morpion_addPlayer(int numSock, long idGame, Games *games)
{
	ListPlayerSockItem *newListPlayerSockItem = NULL, *listPlayerSockItem = NULL;
	ListGameItem *listGameItem = NULL;
	int bcl;

	/* Recherche de la partie ou l'on doit ajouter le joueur */
	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		NOTICE("Game (%d) not exist -> player (%d) didn't added", idGame,
		       numSock);    
		return MORPION_GAMENOEXIST;
	}

	/*
	** Peut-on encore allouer une icone ??
	** ATTENTION, on reserve une icone pour les joueurs qui
	** ont quitte la partie
	*/
	if (listGameItem->game->nbPlayers == (MAXICONNUMBER - 1)){
		NOTICE("Pas d'icone de disponible pour la partie %s(%d)",
		       listGameItem->game->gameName, idGame);
		return MORPION_NOICON;
	}

	/* Recherche de l'existence du joueur */
	listPlayerSockItem = listGameItem->head;
	for (bcl = 0 ; bcl < listGameItem->game->nbPlayers ; bcl++){
		if (listPlayerSockItem->numSock == numSock){
			NOTICE("Player %s(%d) already in game %s(%d)",
			       games->players[numSock].playerName, numSock, 
					 listGameItem->game->gameName, idGame);    
			return MORPION_PLAYERINGAME;
		}
		listPlayerSockItem = listPlayerSockItem->next;
	}

	/*
	** Verification d'une icone disponible pour le joueur et
	** affection de celle-ci
	*/
	if (NULL == (newListPlayerSockItem = (ListPlayerSockItem*) 
					malloc(sizeof(ListPlayerSockItem)))){
		NOTICE("Not enought memory -> Player %s(%d) didn't added", 
		       games->players[numSock].playerName, numSock);    
		return MORPION_ERROR;
	}
	newListPlayerSockItem->numSock = numSock;
	newListPlayerSockItem->next = NULL;

	/*
	** Incrementation du nombre de joueurs et de parties
	** jouees
	*/
	listGameItem->game->nbPlayers++;
	games->players[numSock].nbGamesPlayed++;


	/* Allocation du premier Id de joueur disponible ! */
	if ((newListPlayerSockItem->playerId = 
	     utils_firstBitUnset(listGameItem->game->icon, MAXICONNUMBER)) == -1){
		NOTICE("Plus de creation de joueur possible !!", ".");
		free(newListPlayerSockItem);
		return MORPION_GAMEFULL;
	}
	utils_setBit(newListPlayerSockItem->playerId,
	             &(listGameItem->game->icon), MAXICONNUMBER);

	/* insertion du nouveau joueur avant le joueur courant */
	if (!listGameItem->head){ /* Liste vide*/
		newListPlayerSockItem->next = newListPlayerSockItem;
		listGameItem->head = newListPlayerSockItem;
	} else { /* liste non vide */
		/*
		** On insere le joueur en derniere position
		*/
		for (listPlayerSockItem = listGameItem->head ; listPlayerSockItem->next !=
				listGameItem->head ; listPlayerSockItem = listPlayerSockItem->next);

		newListPlayerSockItem->next = listGameItem->head;
		listPlayerSockItem->next = newListPlayerSockItem;
	}
	NOTICE("Player %s(%d) added in game %s(%d)",
	       games->players[numSock].playerName, numSock, 
			 listGameItem->game->gameName, idGame);    
	return MORPION_OK;
}

/* 
** Function: morpion_updateGrid
**       In: playerId of the player
**           the Game structure.
**   Return: MORPION_OK if the grid has been updated
**           MORPION_GAMENOEXIST if the game don't exist
** Comments: This function modify the grid when a player leave
**           that game. All playerId's are replace by value
**           IDPLAYEROUT. 
*/
valMorpion morpion_updateGrid(int playerId, Game *game)
{
	int bcl;
	
	if (game == NULL){
		NOTICE("Game not exist -> Grid not updated", ".");    
		return MORPION_GAMENOEXIST;
	}

	for (bcl = 0 ; bcl < (game->width * game->height) ; bcl++){
		if (game->grid[bcl] == playerId)
			game->grid[bcl] = IDPLAYEROUT;
	}

	return MORPION_OK;
}

/* 
** Function: morpion_deletePlayerFromGame
**       In: indice of the player in the players array
**           game's Id
**           the main structure containing the Game structure.
**   Return: MORPION_OK if the player has been deleted from game
**           MORPION_PLAYERNOEXIST if the player don't exist in that game
**           MORPION_GAMENOEXIST if the game don't exist
**           If that game has been deleted, a ptr to the next game is
**            return in first parameter (nextGame)
** Comments: This function delete a player to the player's list of the game.
*/
valMorpion morpion_deletePlayerFromGame(ListGameItem** nextGame, int numSock,
                                        long idGame, Games *games)
{
	ListGameItem *listGameItem = NULL;
	ListPlayerSockItem *listPlayerItemIt = NULL, 
		*listPlayerItemDelete = NULL;
    
	/*
	** Recherche de la partie ou l'on doit supprimer le joueur
	*/
	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		NOTICE("Game (%d) not exist -> player (%d) didn't deleted", 
		       idGame, numSock);    
		return MORPION_GAMENOEXIST;
	}

	/*
	** Inititalisation du pointeur sur la partie suivante
	*/
	*nextGame = listGameItem->next;

	/*
	** Si la liste est vide
	*/
	if (!listGameItem->head){
		return MORPION_PLAYERNOEXIST;
	}
	 
	/*
	** Recherche du joueur a supprimer
	*/
	for (listPlayerItemDelete = listGameItem->head ; 
	      listPlayerItemDelete->next !=  listGameItem->head ;
			listPlayerItemDelete = listPlayerItemDelete->next){
		if (listPlayerItemDelete->numSock == numSock)
			break;
	}

	/*
	** Si le joueur a supprimer n'existe pas
	*/
	if (listPlayerItemDelete->numSock != numSock){
		return MORPION_PLAYERNOEXIST;
	}
    
	/* Si la liste ne contient qu'un seul element */
	if (listPlayerItemDelete == listPlayerItemDelete->next){
		free(listPlayerItemDelete);
		listGameItem->head = NULL;
		listGameItem->game->nbPlayers--;
		games->players[numSock].nbGamesPlayed--;
		morpion_deleteGame(idGame, games); /**/
		return MORPION_OK;
	}
    
	/*
	** On se place sur le precedent dans la liste des joueurs
	*/
	for (listPlayerItemIt = listPlayerItemDelete->next ; 
			listPlayerItemIt->next != listPlayerItemDelete ; 
			listPlayerItemIt = listPlayerItemIt->next);
	listPlayerItemIt->next = listPlayerItemDelete->next;
	if (listPlayerItemDelete == listGameItem->head)
	listGameItem->head = listPlayerItemIt;
	/*
	** On remplace tous les coups de ce joueur dans la grille
	** par l'identifiant IDPLAYEROUT. Les coups sont donc garde en memoire
	** et personne ne seront pas recuperes
	*/
	morpion_updateGrid(listPlayerItemDelete->playerId, listGameItem->game);
	utils_unsetBit(listPlayerItemDelete->playerId,
	               &(listGameItem->game->icon), MAXICONNUMBER);
	/*
	** Suppression de la cellule dans la liste sans desallouer le PlayerSock
	** contenu dans la structure ListPlayerSockItem. Ce dernier est utilise
	** ailleurs
	*/
	free(listPlayerItemDelete);
	NOTICE("Player %s(%d) has been deleted from game %s(%d)",
	       games->players[numSock].playerName, numSock, 
	       listGameItem->game->gameName, idGame);
	listGameItem->game->nbPlayers--;
	games->players[numSock].nbGamesPlayed--;
	return MORPION_OK;
}

/* 
** Function: morpion_deletePlayer
**       In: indice of the player in the players array
**           the main structure containing the Game structure.
**   Return: MORPION_OK if the player has been deleted from all the game
**           MORPION_PLAYERNOEXIST if the player don't exist in any game
** Comments: This function delete a player from all the game.
*/
valMorpion morpion_deletePlayer(int numSock, Games *games)
{
	ListGameItem *listGameItem = NULL, *nextGame = NULL;

	/* Si le joueur n'existe pas ou ne joue pas dans une partie */
	if (!games || games->players[numSock].vide || 
	    !games->players[numSock].nbGamesPlayed){
		return MORPION_PLAYERNOEXIST;
	}

	/* Pour chaque partie, on supprime le joueur */
	listGameItem = games->listGames;
	while (games->players[numSock].nbGamesPlayed != 0){
		if ((morpion_deletePlayerFromGame(&nextGame, numSock, 
		      listGameItem->game->idGame, games)) == MORPION_OK)
			listGameItem = nextGame;
		else
			listGameItem = listGameItem->next;
	}
	NOTICE("Player (%d) has been deleted from all the games",
	       numSock);    
	return MORPION_OK;
}

/* 
** Function: morpion_hit
**       In: x coord of the hit
**           y coord of the hit
**           indice of the player in the players array
**           game's id
**           the main structure containing the Game structure.
**   Return: MORPION_OK if the hit is OK
**           MORPION_WIN if the player win
**           MORPION_GAMENOEXIT if the game don't exist
**           MORPION_HITOUTGRID if the hit is out of the grid
**           MORPION_HITKO if the hit has already been played
**           MORPION_PLAYERNOEXIST if the player don't exist in that game
**           MORPION_GRIDFULL if grid is full and noboby win
** Comments: This function play the player's hit in the grid.
**
** format de stockage des donnees :
** Un affichage comme suit (x,y):
**
**       (0,0) (1,0) (2,0)
**       (0,1) (1,1) (2,1)
**       (0,2) (1,2) (2,2)
**
** est stocke comme suit :
**
** (0,0) (1,0) (2,0) (0,1) (1,1) (2,1) (0,2) (1,2) (2,2)
**
*/
valMorpion morpion_hit(int *xCoord, int *yCoord, int numSock,
		       long idGame, Games *games)
{
	ListGameItem *listGameItem = NULL;
	ListPlayerSockItem *listPlayerSockItem = NULL;
	int bcl, playerId;

	/* Verification de l'existance de la partie */
	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		NOTICE("Game (%d) not exist -> HIT from player (%d) impossible",
		       idGame, numSock);    
		return  MORPION_GAMENOEXIST;
	}

	/* Verification de l'existance du joueur dans la partie */
	listPlayerSockItem = listGameItem->head;
	for (bcl = 0 ; bcl < listGameItem->game->nbPlayers ; bcl++){
		if (listPlayerSockItem->numSock == numSock)
			break;
		listPlayerSockItem = listPlayerSockItem->next;
	}
	if (listPlayerSockItem->numSock != numSock){
		NOTICE("Player (%d) not exist in game %s(%d) -> HIT impossible",
		       numSock, listGameItem->game->gameName, idGame);    
		return MORPION_PLAYERNOEXIST;
	}
	playerId = listPlayerSockItem->playerId;

	/* Verification de la coherence des cordonnees */
	if ((*xCoord < 0) || (*xCoord >= listGameItem->game->width ) ||
		 (*yCoord < 0) || (*yCoord >= listGameItem->game->height )){
		return MORPION_HITOUTGRID;
	}

	/* Verification que la case n'a pas ete jouee */
	if ((listGameItem->game->type == MORPION) && 
	    (listGameItem->game->grid[(listGameItem->game->width * *yCoord)
	                              + *xCoord] != -1)){
		return MORPION_HITKO;
	}


	/* Cas du morpion */
	if (listGameItem->game->type == MORPION)
		listGameItem->game->grid[(listGameItem->game->width * *yCoord) +
		                         *xCoord] = playerId;
	/* Cas du puissance 4 */
	else if (listGameItem->game->type == PUISSANCE4){
		for (bcl = listGameItem->game->height - 1 ; bcl >= 0 ; bcl--){
			if (listGameItem->game->grid[(listGameItem->game->width * 
			                             bcl) + *xCoord] == -1)
				break;
		}
		if (listGameItem->game->grid[(listGameItem->game->width * 
			                          bcl) + *xCoord] == -1){
			listGameItem->game->grid[(listGameItem->game->width * bcl)
			                         + *xCoord] = playerId;
			*yCoord = bcl;
		} else{
			return MORPION_MAXHIT;
		}
	}

	/* Incrementation du nb de coups joues dans la partie */
	listGameItem->game->nbHit++;

	/*
	** Si d'autres types de parties sont ajoutees, faire appel
	** a sa fonction qui test l'etat de la partie
	*/
	if (listGameItem->game->type == PUISSANCE4)
		return morpion_statOfGame(*xCoord, *yCoord, playerId, 
		                          listGameItem->game);
	else if (listGameItem->game->type == MORPION)
		return morpion_statOfGame(*xCoord, *yCoord, playerId,
		                          listGameItem->game);
		return MORPION_ERROR;

}

valMorpion morpion_statOfGame(int xCoord, int yCoord, int playerId,
		       Game *game)
{
	int nbDialDesc = 1, nbDialAsc = 1, nbHoriz = 1, nbVert = 1;
	int left, right, bcl, nbAligne;

	/*
	** Si on joue un MORPION, on prend la definition du Morpion
	** pour le nombre de poins alignes (NBALIGNE_MORPION) 
	*/
	if (game->type == MORPION)
		nbAligne = NBALIGNE_MORPION;
	/*
	** Si on joue un PUISSANC4, on prend la definition du Puissance4
	** pour le nombre de poins alignes (NBALIGNE_PUISSANCE4) 
	*/
	else
		nbAligne = NBALIGNE_PUISSANCE4;

	/*
	** Principe :
	**
	*/
   /*
   ** Le joueur a-t-il ou non gagne la partie sur la diagonale
   ** descendante !
   */
   left = 1; /* on continue a gauche */
   right = 1; /* on continue a droite */
   bcl = 1;
   while ((nbDialDesc != nbAligne) && (left || right)){
		if (left && ((xCoord - bcl) >= 0) && ((yCoord - bcl) >= 0)
	       && (game->grid[(yCoord - bcl) * game->width + (xCoord - bcl)]
	       == playerId))
			nbDialDesc++;
		else
			left = 0;
		if (right && ((xCoord + bcl) < game->width) && ((yCoord + bcl)
		    < game->height) && (game->grid[(yCoord + bcl) * game->width
		    + (xCoord + bcl)] == playerId))
			nbDialDesc++;
		else
			right = 0;
		bcl++;
	}
	if (nbDialDesc == nbAligne)
		return MORPION_GAMEWIN;
	    
	/*
	** Calcul du nb de pieces alignees sur la diagonale ascendante
	*/
	right = 1;
	left = 1;
	bcl = 1;
	while ((left || right) &&  (nbDialAsc != nbAligne)){
		if (left && ((xCoord + bcl) < game->width) && ((yCoord - bcl)
		    >= 0) && (game->grid[(yCoord - bcl) * game->width + 
		    (xCoord + bcl)] == playerId))
			nbDialAsc++;
		else
			left = 0;
		if (right && ((xCoord - bcl) >= 0) && ((yCoord + bcl) <
		    game->height) && (game->grid[(yCoord + bcl) * 
			 game->width + (xCoord - bcl)] == playerId))
			nbDialAsc++;
		else
			right = 0;
		bcl++;
	}
	if (nbDialAsc == nbAligne)
		return MORPION_GAMEWIN;
	    
	/*
	** Calcul du nb de pieces alignees sur l'horizontale
	*/
	right = 1;
	left = 1;
	bcl = 1;
	while ((left || right) && (nbHoriz != nbAligne)){
		if (left && ((xCoord - bcl) >= 0) && 
		    (game->grid[yCoord * game->width + (xCoord - bcl)]
		     == playerId))
			nbHoriz++;
		else
			left = 0;
		if (right && ((xCoord + bcl) < game->width) && 
			 (game->grid[yCoord * game->width + (xCoord + bcl)]
			  == playerId))
			nbHoriz++;
		else
			right = 0;
		bcl++;
	}
	if (nbHoriz == nbAligne)
		return MORPION_GAMEWIN;
	    
	/*
	** Calcul du nb de pieces alignees sur la verticale
	*/
	right = 1;
	left = 1;
	bcl = 1;
	while ((left || right) && (nbVert != nbAligne)){
		if (left && ((yCoord + bcl) < game->height) && 
			 (game->grid[(yCoord + bcl) * game->width + xCoord]
			  == playerId))
			nbVert++;
		else
			left = 0;
		if (right && ((yCoord - bcl) >= 0) && (game->grid[(yCoord - bcl) *
			 game->width + xCoord] == playerId))
			nbVert++;
		else
			right = 0;
		bcl++;
	}
	if (nbVert == nbAligne)
		return MORPION_GAMEWIN;
	    
	/*
	** Si la partie n'est pas gagnee
	*/
	if ((game->width * game->height) == game->nbHit){
		NOTICE("GRID IS FULL -> All the player lose", ".");
		return MORPION_GRIDFULL;
	}

	return MORPION_OK;
}

/* 
** Function: morpion_nextPlayer
**       In: indice of the next player in the array of players
**           game's id
**           the main structure containing the Game structure.
**   Return: player's indice in the players array, -1 if the list
**           of players is empty or the game don't exist.
** Comments: This function return the socket of the next player 
**           to play.
*/
int morpion_nextPlayer(long idGame, Games *games)
{
	ListGameItem *listGameItem = NULL;

	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		return -1;
	}

	/* Partie vide !! */
	if (!listGameItem->head)
		return -1;

	listGameItem->head = listGameItem->head->next;
	NOTICE("Next Player from game %s(%d) is %s(%d)",
	       listGameItem->game->gameName, idGame,
          games->players[listGameItem->head->numSock].playerName,
			 listGameItem->head->numSock);
	return listGameItem->head->playerId;
}

/* 
** Function: morpion_playerInGame
**       In: game's id
**           the main structure containing the Game structure.
**   Return: player's indice in the players' Id, -1 if the
**           list of players is empty or the game don't exist.
** Comments: This function return the socket of the current 
**           player to play.
*/
int morpion_playerInGame(long idGame, Games *games)
{
	ListGameItem *listGameItem = NULL;

	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		NOTICE("Game (%d) not exist -> PlayerInGame impossible", idGame);    
		return -1;
	}

	/* Partie vide !! */
	if (!listGameItem->head)
		return -1;

	return listGameItem->head->playerId;
}

/* 
** Function: morpion_infoPlayer
**       In: indice of the player in the players array
**           the main structure containing the Game structure.
**   Return: player's info : {PlayerName} {MachineName} Num_icon
**           NULL if an allocation error occur
** Comments: This function return info on the player.
*/
char* morpion_infoPlayer(ListPlayerSockItem *listPlayerSockItem, Games *games)
{
	char *info = NULL;
	int alloc;
	
	/*
	** Si le joueur n'existe pas
	*/
	if (!games || !listPlayerSockItem){
		return NULL;
	}

	/*
	** retourne l'info sous la forme :
	** {playerId {Nom du joueur} Nom_machine}
	**
	** - 3 caracteres pour son identifiant, le nombre de 
	**   joueur etant au maximum de 256 (SOCKMAX_FD).
	** - strlen(playerName) caracteres pour le nom du joueur.
	** - strlen(machineName) caracteres pour le nom de la machine
	** - 4 caracteres pour les accolades {}.
	** - 2 espaces.
	** - 1 caractere pour le fin de chaine.
	**
	** soit : strlen(playerName) + strlen(machineName) + 10
	** MAX  : MAXPLAYERNAME + MAXMACHINENAME + 10
	*/

	alloc = strlen(games->players[listPlayerSockItem->numSock].playerName) + 
	        strlen(games->players[listPlayerSockItem->numSock].machineName) +
	        10 + 100;
	if ((info = (char*)malloc(sizeof(char) * alloc)) == NULL){
		NOTICE("Erreur sur malloc : ", ".");
		return NULL;
	}

	sprintf(info, "{%d %s %s}", listPlayerSockItem->playerId,
	        games->players[listPlayerSockItem->numSock].playerName,
	        games->players[listPlayerSockItem->numSock].machineName);

	return info; 
}

/* 
** Function: morpion_infoGame
**       In: Game's Id
**           the main structure containing the Game structure.
**   Return: game's info : Game'Id {GameName} width height
**           {infoPlayer1} ... {infoPlayerN}
**           NULL if an allocation error occur
** Comments: This function return info on the game.
*/
char* morpion_infoGame(long idGame, Games *games)
{
	char *info = NULL, *infoPlayer = NULL, stype[15];
	int bcl, alloc;
	ListGameItem *listGameItem = NULL;
	ListPlayerSockItem *listPlayerItemIt = NULL;
    
	/*
	** Recherche de la partie
	*/
	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		return NULL;
	}
    
	/*
	** retourne l'info sous la forme :
	** {IdGame {Nom partie} largeur hauteur { info_joueur1
	**                                       ...
	**                                       info_joueurN }}
	**
	** - 3 caracteres pour le numero de partie
	** - strlen(gameName) caracteres pour le nom de la
	**   partie.
	** - 6 caracteres pour les dimensions de la grille.
	** - (MAXPLAYERNAME + MAXMACHINENAME + 10) * nbJoueurs 
	**   caracteres pour les infos des joueurs
	** - 6 + (nbJoueurs - 1) espaces
	** - 6 accolades
	** - 1 caractere pour le fin de chaine.
	**
	** soit : (MAXPLAYERNAME + MAXMACHINENAME + 10) * nbJoueurs +
	**        strlen(gameName) + 22 + (nbJoueurs - 1)
	** MAX  : (MAXPLAYERNAME + MAXMACHINENAME + 10) * nbJoueurs +
	**        MAXGAMENAME + 22 + (nbJoueurs - 1)
	*/
	alloc = (MAXPLAYERNAME + MAXMACHINENAME + 10) * listGameItem->game->nbPlayers
	        + strlen(listGameItem->game->gameName) + 22 +
			  listGameItem->game->nbPlayers + 100;
	if ((info = (char*) malloc(sizeof(char) * alloc)) == NULL){
		NOTICE("Erreur sur malloc : ", ".");
		return NULL;
	}
 
	strncpy(stype, partie[listGameItem->game->type], 15);
	sprintf(info, "{%ld %s %s %d %d {", listGameItem->game->idGame,
	        listGameItem->game->gameName, stype,
	        listGameItem->game->width, 
	        listGameItem->game->height);
	listPlayerItemIt = listGameItem->head;

	for (bcl = 0 ; bcl < listGameItem->game->nbPlayers ; bcl++){
		infoPlayer = morpion_infoPlayer(listPlayerItemIt, games);
		sprintf(info, "%s %s", info, infoPlayer);
		free(infoPlayer);		       
		listPlayerItemIt = listPlayerItemIt->next;
	}
	sprintf(info, "%s }}", info);

	return info;
}

/* 
** Function: morpion_infoGames
**       In: the main structure containing the Game structure.
**   Return: info on all the game :
**           {{Game'Id {GameName1} width height {infoPlayer1}
**           ... {infoPlayerN}} ...  {Game'Id {GameNameM} width
**           height {infoPlayer1} .. {infoPlayerM}}}
**           NULL if an allocation error occur
** Comments: This function return info on ALL the game.
*/
char* morpion_infoGames(Games *games)
{
	char *info = NULL, *infoGame = NULL;
	ListGameItem *listGameItem = NULL;
	int alloc = 0;

	/*
	** Calcul de la taille a allouer pour les infos
	**
	** retourne l'info sous la forme :
	** { info_partie1 ... info_partieN }
	**
	** Pour chaque partie on a :
	** - ((MAXPLAYERNAME + MAXMACHINENAME + 10) * nbJoueurs +
	**   MAXGAMENAME + 22 + (nbJoueurs - 1)) * nbParties
	** - 2 caracteres pour les acolades.
	** - (nbParties - 1) + 2 espaces.
	** - 1 caractere de fin de chaine.
	**
	*/

	alloc = 0;
	for (listGameItem = games->listGames ; listGameItem ;
			listGameItem = listGameItem->next)
		alloc = alloc + (MAXPLAYERNAME + MAXMACHINENAME + 10) *
		        listGameItem->game->nbPlayers + MAXGAMENAME + 22 +
				  (listGameItem->game->nbPlayers - 1) + 2;
	alloc = alloc + 5000;
	if ((info = (char*) malloc(sizeof(char) * alloc)) == NULL){
		NOTICE("Erreur sur malloc : ", ".");
		return NULL;
	}

	sprintf(info ,"{");
	for (listGameItem = games->listGames ; listGameItem ;
			listGameItem = listGameItem->next){
		infoGame = morpion_infoGame(listGameItem->game->idGame, games);
		sprintf(info, "%s %s", info, infoGame);
		free(infoGame);
	}
	sprintf(info,"%s }", info);

	return info;
}

/* 
** Function: morpion_grid
**       In: Game's Id
**           the main structure containing the Game structure.
**   Return: info on the grid :
**           {playerId1 {x1 y1} {x2 y2} .. {xn yn}}
**            ...
**           {playerIdN {x1 y1} {x2 y2} .. {xn yn}}
**           NULL if an allocation error occur
** Comments: This function return hits on a grid.
*/
char* morpion_grid(long idGame, Games *games){
	char *info = NULL;
	ListGameItem *listGameItem = NULL;
	Game *game = NULL;
	ListPlayerSockItem *player = NULL;
	int alloc = 0,     /* Size to allocate */
		x = 0,
		y = 0,
		bclPlayer = 0, /* indice of player in game */
		bclGrid = 0,   /* indice in grid */
		dimGrid = 0;   /* Grid dimension */
  
	/*
	** Recherche de la partie
	*/
	if ((listGameItem = morpion_existGameNum(idGame, games)) == NULL){
		NOTICE("Game (%d) not exist : ", idGame);
		return NULL;
	}

	game = listGameItem->game;
	dimGrid = game->width * game->height;

	/*
	** Calcul de la taille a allouer pour les donnees de la grille
	**
	** retourne la grille sous la forme :
	** {Player1 {x1 y1} {x2 y2} .. {xn yn}}
	** {Player2 {x1 y1} {x2 y2} .. {xn yn}}
	**
	** Pour chaque partie on a :
	** - 3 * nbPlayers caracteres pour les icones.
	** - 9 caracteres pour chaque coup (2 par coord + ' ' + {})
	** - 2 caracteres pour les {} par joueur
	** - nbjoueurs + 1 espaces entre les joueurs
	** - (nb coups) espaces entre chaque coups.
	** - 1 car pour '\0'
	**
	** Soit : ((3 + 2) * nbplayers) + ((9 + 1) * nbHit)
	**        + (nbjoueurs + 1 ) + 1
	*/

	alloc = (game->nbPlayers * 5) + (10 * game->nbHit) + game->nbPlayers + 4 +
	         1000;
	if ((info = (char*) malloc(sizeof(char) * alloc)) == NULL){
		NOTICE("Erreur sur malloc : ", ".");
		return NULL;
	}

	sprintf(info, "%ld ", idGame);

	/*
	** Cas des coups associes au joueurs qui ont quitte
	** une partie
	*/
	sprintf(info, "%s {%d", info, IDPLAYEROUT);
	for (bclGrid = 0 ; bclGrid < dimGrid ; bclGrid ++){
		if (game->grid[bclGrid] == IDPLAYEROUT){
			y = bclGrid / game->width;
			x = bclGrid - (game->width * y);
			sprintf(info, "%s {%d %d}", info, x, y);
		}
	}
	sprintf(info, "%s}", info);
	/* Cas general */
	player = listGameItem->head;
	for (bclPlayer = 0 ; bclPlayer < game->nbPlayers ; bclPlayer++){
		sprintf(info, "%s {%d", info, player->playerId);
		for (bclGrid = 0 ; bclGrid < dimGrid ; bclGrid ++){
			if (game->grid[bclGrid] == player->playerId){
				y = bclGrid / game->width;
				x = bclGrid - (game->width * y);
				sprintf(info, "%s {%d %d}", info, x, y);
			}
		}
		sprintf(info, "%s}", info);
		player = player->next;
	}

	return info;
}
