/* -*-linux-c-*-
** $Id: libutils.c,v 1.2 1998/04/24 15:52:34 enlight Exp $
** $Log: libutils.c,v $
** Revision 1.2  1998/04/24 15:52:34  enlight
** Probleme d'inversion du nom d'une des fonctions
**
** Revision 1.1  1998/04/24 15:39:52  enlight
** Initial revision
**
*/
#include "libutils.h"




/* 
** Function: utils_initBit
**       In: the array of bit.
**           the nb of bits in the array
**   Return: Nothing
** Comments: This function initialise the bits of array to 0.
*/
void utils_initBit(char **tab, int len)
{
	int bcl, maxLen;

	if ((len % OCTET) != 0)
		maxLen = len / OCTET + 1;
	else
		maxLen = len / OCTET;
		
	for (bcl = 0 ; bcl < maxLen ; bcl++)
		(*tab)[bcl] = 0;

}

/* 
** Function: utils_firstBitUnset
**       In: the array of bit
**           the nb of bits in the array
**   Return: the first bit unset.
**           -1 if all bits are set
** Comments: This function return the first unset bit in an 
**           array.
*/
int utils_firstBitUnset(char *tab, int len)
{
	int bcl;
	
	for (bcl = 0 ; bcl < len ; bcl++){
		if (((tab[bcl / OCTET] & (1 << bcl % OCTET)) >> (bcl % OCTET)) == 0)
			return bcl;
	}
	
	/* No bit free */
	return -1;
}

/* 
** Function: utils_lastBitSet
**       In: the array of bit
**           the nb of bits in the array
**   Return: the last bit set.
**           -1 if all bits are unset
** Comments: This function return the last set bit in an 
**           array.
*/
int utils_lastBitSet(char *tab, int len)
{
	int bcl;
	
	for (bcl = len - 1 ; bcl >= 0 ; bcl--){
		if (((tab[bcl / OCTET] & (1 << bcl % OCTET)) >> (bcl % OCTET)) == 1)
			return bcl;
	}
	
	/* No bit Set */
	return -1;
}


/* 
** Function: utils_isBitSet
**       In: the bit to test
**           the array of bit
**           the nb of bits in the array
**   Return: UTILS_OUTOFRANGE if the bit to test is out of range
**           UTILS_SET if the bit is set
**           UTILS_UNSET else
** Comments: This function test if a bit is set in an array.
*/
int utils_isBitSet(int bit, char *tab, int len)
{
	if (len < bit)
		return UTILS_OUTOFRANGE;

	if ((tab[bit / OCTET] & (1 << (bit % OCTET))) >> (bit % OCTET))
		return UTILS_SET;
	else
		return UTILS_UNSET;
}

/* 
** Function: utils_setBit
**       In: the bit to set
**           the array of bit.
**           the nb of bits in the array
**   Return: UTILS_OUTOFRANGE if the bit to set is out of range
**           UTILS_OK else.
**           modify the array of bits (tab).
** Comments: This function set a bit in an array.
*/
int utils_setBit(int bit, char **tab, int len)
{
	if (len < bit)
		return UTILS_OUTOFRANGE;

	(*tab)[bit / OCTET] |= 1 << (bit % OCTET);
	return UTILS_OK;
}

/* 
** Function: utils_unsetBit
**       In: the bit to unset
**           the array of bit.
**           the nb of bits in the array
**   Return: UTILS_OUTOFRANGE if the bit to unset is out of range
**           UTILS_OK else
**           modify the array of bits (tab).
** Comments: This function unset a bit in an array.
*/
int utils_unsetBit(int bit, char **tab, int len)
{
	if (len < bit)
		return UTILS_OUTOFRANGE;

	(*tab)[bit / OCTET] &= ~(1 << (bit % OCTET));
	return UTILS_OK;
}

