/* -*-linux-c-*-
** $Id: serveur.c,v 1.3 1998/04/23 03:31:00 enlight Exp enlight $
** $Log: serveur.c,v $
** Revision 1.3  1998/04/23 03:31:00  enlight
** Ajout support de l'evenement EVENT_LOSTCONNEX.
** Ajout de l'appel a l'initialisation des structures de l'appli.
** Ajout de l'affichage du rcsid a chaque lancement.
**
** Revision 1.2  1998/04/22 19:13:47  enlight
** Rajoute identification au lancement du serveur
**
** Revision 1.1  1998/04/22 19:06:19  enlight
** Initial revision
**
*/

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include "global.h"
#include "servMorp.h"
#include "libconnex.h"
#include "libutils.h"
#include "libsecur.h"
#include "libprotocol.h"
#include "libmorpion.h"

#ifndef S_ISLNK
#define S_ISLNK(m)      (((m) & S_IFMT) == S_IFLNK)
#endif

extern int errno;

void usage(const char *str)
{
    fprintf(stderr, "usage: %s [ options ]\n ", str);
    fprintf(stderr, "\n\
This program is the daemon server for the Morpin game.\n\
Options include:\n\
\n\
      -h or --help            this message\n\
      -p or --port <port>     port to use (default: %d)\n\
      -n or --nodaemon        run the server without detaching\n\
      -V or --version         gives program version\n\
\n\n", MONPORT);

    exit(EXIT_SUCCESS);
}

void version(void)
{
    fprintf(stderr, "Serveur de morpion "PROG" v"VERSION"\n");
    fprintf(stderr, "(c) Sebastien Moreau, Luc Stepniewski et Pierre Vassellerie\n");
    fprintf(stderr, "===========================================================\n\n");
    fprintf(stderr, "max possible open files on this sytem = %d\n", SOCKMAX_FD);
    exit(EXIT_SUCCESS);
}

int main(int argc, char *argv[])
{
    int i, retCode, fd, port = 0, doNotDetach = 0, nbGamesPlayed = 0;
    char strPid[50];      /* Buffer to store the pid of the current process */
    char lineBuf[1024], toString[1024], *strToSend, *listOfGames, *gridToSend;
    FILE *fMotd;
    Sockets sockets;
    SockEvent *ev;
    struct stat buf;

    /* Traitement de la ligne de commande */
    for (i = 1; i < argc; i++) {
        const char *sw = argv[i];
        /* On passe les premiers car inutiles */
        if ((sw[0] == '-') && (sw[1] == '-'))
            sw++;

        if (!strcmp(sw, "-h") || !strcmp(argv[i], "-help"))
            usage(argv[0]);
        else if (!strcmp(sw, "-p") || !strcmp(sw, "-port")){
            if ((i+1) >= argc)
                usage(argv[0]);
            port = atoi(argv[++i]);
        }
        else if (!strcmp(sw, "-V") || !strcmp(sw, "-version"))
            version();
        else if (!strcmp(sw, "-n") || !strcmp(sw, "-nodaemon"))
            doNotDetach = 1;
        else if (sw[0] == '-' && sw[1])
            usage(argv[0]);
    }
    
    /* Securisation de l'environnement */
    secur_nettoieEnv();

    /* Initialisation des variables et structures de l'application */
    morpion_init(&sockets);

    /* Let's become a daemon */
    socket_initDaemon(doNotDetach);

    /* From here, we're no more connected to anything ;-) */

    /* Initialisation des structures et des sockets */
    socket_setup(&sockets, port);

    /* Verification que personne n'essaye de faire de betises ... */
    retCode = lstat("/tmp/titbetd.pid", &buf);
    if (retCode != -1){
        if (S_ISLNK(buf.st_mode)){
            NOTICE("Tentative de subversion du serveur detectee, par (%d, %d)",
                   buf.st_uid, buf.st_gid);
            exit(EXIT_FAILURE);
        }
    } else if (errno != ENOENT){
        NOTICE("Erreur execution lstat ;-(\n", ".");
        exit(EXIT_FAILURE);
    }
    
    /* ajout du pid dans le fichier */
    if ((fd = creat("/tmp/titbetd.pid", 
                    S_IRUSR|S_IRGRP|S_IROTH|S_IWUSR)) != -1){
        sprintf(strPid, "%d", (int) getpid());
        write(fd, strPid, strlen(strPid));
        close(fd);
    }

    while(1){
        ev = socket_waitEvent(&sockets);
        if (ev == NULL){
            NOTICE("Abnormal server behaviour. ev == NULL\n", ".");
            kill(getpid(), SIGTERM);
            exit(EXIT_FAILURE);
        }
        
	if (ev->type == EVENT_NEWCONNEX){
            socket_print(ev->fd, "MSG Serveur "PROG" v"VERSION"\n");
	    socket_print(ev->fd, "MSG (c) Sebastien Moreau,  Luc Stepniewski et Pierre Vassellerie\n");

	    if ((fMotd = fopen(PATHTOMOTD, "r")) != NULL){
		while (!feof(fMotd)){
			if (fgets(lineBuf, 1000, fMotd) == NULL)
				break;
			sprintf(toString, "MSG %s", lineBuf);
			socket_print(ev->fd, toString);
		}
		fclose(fMotd);
	    } else {
	    	if(socket_print(ev->fd, 
                                "MSG Bienvenue sur le serveur\n") == -1){
			NOTICE("Pas pu envoyer les donnees", ".");
            	}
	    }
	} else            
            if (ev->type == EVENT_REPLY){
                /* 
                ** Received a message from one of the clients
                */
                proto_evalMessage(ev);
            } else 
                if (ev->type == EVENT_LOSTCONNEX){
                    /* Erase the player from all of the games... */
                    retCode = morpion_deletePlayer(ev->fd, &(sockets.games));
                    if ((retCode != MORPION_OK) && 
                        (retCode != MORPION_PLAYERNOEXIST)){
                        NOTICE("FATAL Erreur morpion_playerDelete ->(%d)", 
                               retCode);
                        kill(getpid(), SIGTERM);
                    }
                    
                    /* ...And notify everyone that the player has gone */
                    listOfGames = morpion_infoGames(&(sockets.games));
                    
                    strToSend = (char *)malloc(sizeof(char) * 
                                               (strlen(listOfGames) + 15));
                    sprintf(strToSend, "GAMES %s\n", listOfGames);
                    morpion_noticeToAllPlayers(-1, -1, 
                                               BROADCAST_ALLPLAYERS,
                                               strToSend, &(sockets.games));
                    free(strToSend);
                    free(listOfGames);
                    
                    nbGamesPlayed = 
                        utils_lastBitSet(sockets.games.listIdGame, 
                                         LISTIDGAME);
                    for(i=0; i < nbGamesPlayed; i++){
                        gridToSend = morpion_grid(i, &(sockets.games));
			if (gridToSend != NULL){
                            strToSend = (char *)malloc(sizeof(char) *
                                                    (strlen(gridToSend) + 10));
                            sprintf(strToSend, "GRID %s\n", gridToSend);
                            morpion_noticeToAllPlayers(-1, i,
                                                   BROADCAST_GAME,
                                                   strToSend, &(sockets.games));
                            free(strToSend);
                            free(gridToSend);
			}
                    }

                    morpion_playerReInit(ev->fd, &sockets);
                    NOTICE("Effacement du joueur %d (ret=%d)",
                           ev->fd, retCode);

                } else
                    if ((ev->type == EVENT_ERROR) ||
                        (ev->type == EVENT_BADDESCRI) ||
                        (ev->type == EVENT_INTERRUPT)){
                        NOTICE("Probleme renvoye dans l'evenement(%d)", 
                               ev->type);
                    } else
                        NOTICE("Empty event received", ".");
        
        /* Let's free that structure */
        free(ev);
    }
    
    /* End of the main program */
    return EXIT_SUCCESS;
}



